package dybip;

import java.util.ArrayList;

import dybip.componentInstance.DataMap;
import dybip.expression.Expression;
import dybip.expression.list.DataList;
import dybip.motifInstance.MotifArrayListMap;
import dybip.motifInstance.MotifInstance;

public abstract class Context extends DyBIPObject{
	private IndexMap indexMap = new IndexMap(); // used to store indexes in interaction rules being interpreted
	// keep track of current motifs when evaluating for each rules
	private MotifArrayListMap mlistmap = new MotifArrayListMap();
	private DataMap dataMap = new DataMap();
	abstract public Expression find(String name);

	public void setIndexMap(IndexMap indexMap){
		this.indexMap = indexMap;
	}
	public IndexMap getIndexMap(){
		return indexMap;
	}
	public DataMap getDataMap() {
		return dataMap;
	}
	public MotifArrayListMap getMotifArrayListMap() {
		return mlistmap;
	}
	public void addMotifToCurrentScope(String varName, MotifInstance minstance) {
		ArrayList<MotifInstance> motifArray = new ArrayList<MotifInstance>(1);
		motifArray.add(minstance);
 		mlistmap.put(varName, motifArray);
	}
	public void setDataMap(DataMap map) {
		dataMap = map;
	}
	public void resetDataMap() {
		dataMap = new DataMap();
	}
	public void resetIndexMap() {
		indexMap = new IndexMap();
	}
	public void resetMotifListMap() {
		mlistmap = new MotifArrayListMap();
	}
	public ArrayList<MotifInstance> getMotifListWithName(String varName) {
		if(mlistmap.containsKey(varName))
			return mlistmap.get(varName);
		else
			return null;
	}
	public void putMotifList(String varName, ArrayList<MotifInstance> motifList) {
		mlistmap.put(varName, motifList);
	}
	public Integer getIndex(Expression indexExpr) {
		Expression minifiedIndexExpr = indexExpr.minify(this);
		Integer index = -1;
		if(minifiedIndexExpr instanceof DataList ){
			DataList dlist = (DataList) minifiedIndexExpr;
			if(dlist.getListType(this) == Integer.class)
			{
				if(dlist.size() == 1)
					index = dlist.get(0).minify(this).value(Integer.class);
				else ;
					// TODO given a list of integers return a new group with all indexed elements 
					// multiple array aceess;	
			}
			else
				System.err.println("Cant have a non-integer index to a group");
			
		}
		else // index either an integer or a loop counter
			index = minifiedIndexExpr.value(Integer.class);
		
		return index;
	}
	public void putIndex(String key, Integer elem){
		indexMap.put(key, elem);
	}
}
