package dybip;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;
import java.util.Scanner;

import miscellaneous.PrintColorWriter;

import org.antlr.v4.runtime.*;
import org.antlr.v4.runtime.tree.*;
import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;

import dybip.expression.Expression;
import dybip.system.SystemInstance;

import antlr4.*;

public class DyBIPRunner {
	
	
	public static final int DEFAULTMOVECOUNT = 100;
	private static int moveCount = DEFAULTMOVECOUNT;
	
	public static final int DEFAULTSYSTEMRUNCOUNT = 10;
	private static int systemRunCount = DEFAULTSYSTEMRUNCOUNT; 

	public static final boolean RANDOM = true;
	public static final boolean USERDRIVEN = false;
	private static boolean moveMode = RANDOM;
	
	public static final int VERBOSE = 0;
	public static final int DEBUG = 1;
	public static final int NORMAL = 2;
	public static final int TRACE = 3;
	private static int toStringMode = NORMAL;

	private static String inputFilePath;
	
	private static ParameterMap parameterMap = new ParameterMap();
	private static MotifInstanceCountMap motifInstanceCountMap = new MotifInstanceCountMap();
	
	private static PrintColorWriter out; 
	public static void main(String[] args) throws Exception {
		try{
			out = new PrintColorWriter(java.lang.System.out);
		}catch (UnsupportedEncodingException e) {
			e.printStackTrace();
			System.exit(1);
		}
		Options options = getCommandLineInterfaceOptions();
		parseOptions(options, args);
		
		String inputFile = inputFilePath;
		InputStream is = new FileInputStream(inputFile);

		ANTLRInputStream inputStream = new ANTLRInputStream(is);
		DyBIPLexer lexer = new DyBIPLexer(inputStream);
		CommonTokenStream tokens = new CommonTokenStream(lexer);
		DyBIPParser dybipParser = new DyBIPParser(tokens);
		
		ParseTree tree = dybipParser.system(); // begin parsing at init rule
		DyBIPTreeVisitor treeVisitor = new DyBIPTreeVisitor();
		dybip.systemStmt.SystemStmt  sysStmt = treeVisitor.visit(tree).asSystem();
	
		PrintWriter statWriter = setUpStatFile(inputFile);
		PrintWriter sizeWriter = setUpSizeFile(inputFile);
//		PrintWriter traceWriter = setUpTraceFile(inputFile);

		
		for(int i = 0; i < systemRunCount; i++) {
			System.out.println("----------------run #" + i);
			//traceWriter.println("[DRBIP ENGINE]: initialize components...");
			dybip.system.SystemInstance sysInstance = new SystemInstance(sysStmt,  motifInstanceCountMap, parameterMap);
			//traceWriter.println("[DRBIP ENGINE]: \n" + sysInstance.getTrace());

			boolean statsForThisRun = i == 0; // detail all stats for a single 1000 step 
			sysInstance.run(i, moveMode, moveCount, toStringMode, statWriter, sizeWriter, statsForThisRun);
			
			parameterMap.update();
		}
		statWriter.close();
		//traceWriter.close();		
	}
	
//	private static PrintWriter setUpTraceFile(String inputFile) throws IOException {
//		File traceFile = new File("./examples/traces/trace_" + getFullFileName(inputFile)+ ".txt");
//		traceFile.createNewFile();
//		PrintWriter traceWriter = new PrintWriter(traceFile);
//		traceWriter.println("[DRBIP ENGINE]: DRBIP Engine started\n[DRBIP ENGINE]:");
//		return traceWriter;
//	}
	private static PrintWriter setUpStatFile(String inputFile) throws IOException {
		
		File statFile = new File("./examples/statistics/stats_" + getFileName(inputFile) + ".txt");
		statFile.createNewFile();
		PrintWriter statWriter = new PrintWriter(statFile);
		
		statWriter.println("File:\t" + inputFilePath.substring(inputFilePath.lastIndexOf('/')+1, inputFilePath.lastIndexOf('.')) + "\n" +
							"Steps:\t" + moveCount +  
							"\nRuns:\t" + systemRunCount);
		for(String morifType: motifInstanceCountMap.keySet())
			statWriter.println(morifType + ":\t" + motifInstanceCountMap.get(morifType).toString());
		for(String param: parameterMap.keySet())
			statWriter.println(param + ":\t" + parameterMap.get(param).toString());
		
		return statWriter;
	}
	private static PrintWriter setUpSizeFile(String inputFile) throws IOException {
		
		File statFile = new File("./examples/statistics/size_" + getFileName(inputFile) + ".txt");
		statFile.createNewFile();
		PrintWriter statWriter = new PrintWriter(statFile);
		
		statWriter.println("File:\t" + inputFilePath.substring(inputFilePath.lastIndexOf('/')+1, inputFilePath.lastIndexOf('.')) + "\n" +
							"Steps:\t" + moveCount +  
							"\nRuns:\t" + systemRunCount);
		for(String morifType: motifInstanceCountMap.keySet())
			statWriter.println(morifType + ":\t" + motifInstanceCountMap.get(morifType).toString());
		for(String param: parameterMap.keySet())
			statWriter.println(param + ":\t" + parameterMap.get(param).toString());
		
		return statWriter;
	}
	private static String getFileName(String inputFilePath)
	{
		String fileName = inputFilePath.substring(inputFilePath.lastIndexOf('/')+1, inputFilePath.lastIndexOf('.'));
		fileName += "_s" + moveCount+ "_r" + systemRunCount;
		
		// parse motif instance count from cmd
		for(String morifType: motifInstanceCountMap.keySet())
			fileName += "_" + morifType.charAt(0) + motifInstanceCountMap.get(morifType).toString();
		
		// parse parameters
		for(String param: parameterMap.keySet())
			fileName += "_" + param + parameterMap.get(param).toString();
		
		return fileName;
	}
	
	private static void parseOptions(Options options, String [] args) {
		CommandLineParser cmdParser = new DefaultParser();
		CommandLine cmd;

		try {
			cmd = cmdParser.parse(options, args);
		} catch (ParseException e) {
			HelpFormatter formatter = new HelpFormatter();
			System.err.println(e.getMessage());
			formatter.printHelp("dybip", options);
			System.exit(1);
			return;
		}
		inputFilePath = cmd.getOptionValue("input");
		
		if(cmd.hasOption("s")){ // step coount
			String moveCountStr = cmd.getOptionValue("s");
			if(isInteger(moveCountStr)) moveCount = Integer.parseInt(moveCountStr);
			else{out.printRedln("\"" + moveCountStr + "\" is not a valid value for option -s. value must be an integer");
				System.exit(1);}
		}
		if(cmd.hasOption("r")) {
			String runCountStr = cmd.getOptionValue("r");
			if(isInteger(runCountStr)) systemRunCount = Integer.parseInt(runCountStr);
			else {out.printRedln("\"" + runCountStr + "\" is not a valid value for option -run. value must be an integer");
			System.exit(1);}
		}
		if(cmd.hasOption("rdm")){
			String randomMove = cmd.getOptionValue("rdm");
			if(isBoolean(randomMove))moveMode = Boolean.parseBoolean(randomMove); 
			else{out.printRedln("\"" + randomMove + "\" is not a valid value for option -rdm. value must be a boolean");
				System.exit(1);}
		}
		
		// TODO both debug and verbose option allowed?
		if(cmd.hasOption("debug"))
			toStringMode = DEBUG;
	
		if(cmd.hasOption("m")){
			String [] str= cmd.getOptionValues("m");
			for(int i = 0; i < str.length; i+=2) {
				ANTLRInputStream inputStream = new ANTLRInputStream(str[i+1]);
				DyBIPLexer lexer = new DyBIPLexer(inputStream);
				CommonTokenStream tokens = new CommonTokenStream(lexer);
				DyBIPParser dybipParser = new DyBIPParser(tokens);
				ParseTree tree = dybipParser.expr();
				
				DyBIPTreeVisitor treeVisitor = new DyBIPTreeVisitor();
				Expression  expr = treeVisitor.visit(tree).asExpression();
				
				motifInstanceCountMap.put(str[i], expr);
			}
			
		}
		
		if(cmd.hasOption("p")) {
			String [] str= cmd.getOptionValues("p");
			for(int i = 0; i < str.length; i+=3) 
				parameterMap.put(str[i], new ParameterIncrement(str[i+1], str[i+2]));	
		}

	}
	
	private static Options getCommandLineInterfaceOptions(){
		Options options = new Options();
	
		Option input = new Option("i", "input", true, "input file path");
		input.setRequired(true);
		options.addOption(input);

		Option moveCount = new Option("s", "step-per-run", true, "count of steps the system makes per run");
		moveCount.setRequired(false);
		moveCount.setType(Number.class);
		options.addOption(moveCount);
		
		Option runCount = new Option("r", "run-count", true, "count of runs the system makes");
		runCount.setRequired(false);
		runCount.setType(Number.class);
		options.addOption(runCount);
		
		Option randomMove = new Option("rdm", "random-move", true, "system picks random interaction if true, otherwise user picks the next Step");
		moveCount.setRequired(false);
		moveCount.setType(Boolean.class);
		options.addOption(randomMove);
		
		Option debug = new Option("d", "debug", false, "print debugging logs");
		moveCount.setRequired(false);
		options.addOption(debug);
				
		Option help = new Option("h", "help", false, "print help message");
		moveCount.setRequired(false);
		options.addOption(help);
		
		Option motif = new Option("m", "motif", true, "set count of motif instances");
		motif.setRequired(false);
		motif.setArgs(Option.UNLIMITED_VALUES);
		options.addOption(motif);
		
		Option variableParam = new Option("p", "parameter", true, "set parameter");
		variableParam.setArgs(Option.UNLIMITED_VALUES);
		variableParam.setRequired(false);
		options.addOption(variableParam);
				
		return options;
	}
	private static boolean isBoolean(String s){
		s = s.toLowerCase();
		if(s.equals("true") || s.equals("false"))
			return true;
		else return false;
	}
	private static boolean isInteger(String s) {
	    Scanner sc = new Scanner(s.trim());
	    if(!sc.hasNextInt()){ 
	    	sc.close();
	    	return false;
	    }
	    // we know it starts with a valid int, now make sure
	    // there's nothing left!
	    sc.nextInt();
	    Boolean result = !sc.hasNext();
		sc.close();
	    return result;
	}
}

//if (cmd.hasOption("fixed-motif")) {
//String [] props = cmd.getOptionValues("fm");
//paramaterMap.put(props[0], props[1]);
//}
//if (cmd.hasOption("variable-motif")) {
//String [] props = cmd.getOptionValues("vm");
//variableParamMap.put(props[0], new VariableParameter(props[1], props[2]));	
//paramaterMap.put(props[0], props[1]);
//}




//		Future<JDialog> futureDialog = parser.system().inspect(Arrays.asList(parser.getRuleNames()));
//		// wait for the dialog to close (if you want)
//		Utils.waitForClose(futureDialog.get());

//		System.out.println(tree.toStringTree(parser));
