package dybip;


import antlr4.*;
import antlr4.DyBIPParser.*;
import dybip.InteractionStmt.ForEachIterator;
import dybip.InteractionStmt.ForIterator;
import dybip.InteractionStmt.InterReconfigurationRuleStmt;
import dybip.InteractionStmt.InterReconfigurationRuleStmtList;
import dybip.InteractionStmt.InteractionRuleStmtList;
import dybip.InteractionStmt.NestedIterators;
import dybip.InteractionStmt.InteractionRuleStmt;
import dybip.InteractionStmt.Range;
import dybip.InteractionStmt.ReconfigurationRuleStmt;
import dybip.InteractionStmt.ReconfigurationRuleStmtList;
import dybip.componentType.PortSymbolMap;

import dybip.expression.*;
import dybip.expression.action.*;
import dybip.expression.binary.*;
import dybip.expression.booleanExpr.*;
import dybip.expression.list.ActionList;
import dybip.expression.list.GuardList;
import dybip.expression.literal.*;
import dybip.expression.unary.*;
import dybip.expression.variable.*;
import dybip.systemStmt.SystemStmt;
import dybip.systemStmt.componentTypeStmt.ComponentTypeStmt;
import dybip.systemStmt.componentTypeStmt.ComponentTypeStmtList;
import dybip.systemStmt.componentTypeStmt.DataTypeStmt;
import dybip.systemStmt.componentTypeStmt.DataTypeStmtList;
import dybip.systemStmt.componentTypeStmt.InitTransitionStmt;
import dybip.systemStmt.componentTypeStmt.PortStmt;
import dybip.systemStmt.componentTypeStmt.PortStmtList;
import dybip.systemStmt.componentTypeStmt.StateStmt;
import dybip.systemStmt.componentTypeStmt.TransitionStmt;
import dybip.systemStmt.componentTypeStmt.TransitionStmtList;
import dybip.systemStmt.motifInstanceStmt.MotifInstanceStmt;
import dybip.systemStmt.motifInstanceStmt.MotifInstanceStmtList;
import dybip.systemStmt.motifTypeStmt.ComponentInstanceStmt;
import dybip.systemStmt.motifTypeStmt.ComponentInstanceStmtList;
import dybip.systemStmt.motifTypeStmt.MotifTypeStmt;
import dybip.systemStmt.motifTypeStmt.MotifTypeStmtList;
import dybip.systemStmt.motifTypeStmt.PortSymbolList;
import dybip.systemStmt.motifTypeStmt.ReferStmt;
import dybip.systemStmt.motifTypeStmt.ReferStmtList;
public class DyBIPTreeVisitor extends DyBIPBaseVisitor<DyBIPObject>{

	dybip.systemStmt.SystemStmt system = null;
	
	@Override
	public DyBIPObject visitSystem(DyBIPParser.SystemContext ctx) {

		//-----sys name
		String name = ctx.ID().getText();

		//----sys component TYpes
		int compTypeCount = ctx.componentType().size();
		ComponentTypeStmtList ctypes = new ComponentTypeStmtList(compTypeCount);
		for(int i = 0; i < compTypeCount; i++)
			ctypes.add(visit(ctx.componentType(i)).asComponentTypeStmt());

		
		// --- motif types
		int motifTypeCount = ctx.motifType().size();
		MotifTypeStmtList mtypes = new MotifTypeStmtList(motifTypeCount);
		for(int i = 0; i < motifTypeCount; i++)
			mtypes.add(visit(ctx.motifType(i)).asMotifTypeStmt());
		
		
		// -- System interaction rules
		int rulesCount = ctx.interReconfigurationRule().size();
		InterReconfigurationRuleStmtList interReconfigurationRuleList = new InterReconfigurationRuleStmtList();
		for(int i = 0; i < rulesCount; i++){
			DyBIPObject rule = visit(ctx.interReconfigurationRule(i));
			if(rule instanceof InterReconfigurationRuleStmt)
				interReconfigurationRuleList.add(rule.asInterReconfigurationRuleStmt()); 
		}
		
		
		// -- motif instances
		int motifInstanceCount = ctx.motifInstance().size();
		MotifInstanceStmtList motifInstanceList = new MotifInstanceStmtList(motifInstanceCount);
		for(int i = 0; i < motifInstanceCount; i++){
			motifInstanceList.add(visit(ctx.motifInstance(i)).asMotifInstanceStmtList());
		}
		
		system = new SystemStmt(name, ctypes, mtypes, interReconfigurationRuleList,  motifInstanceList);
		return system;
	}
	/// -------------------------------------------------- Component Type ----------------------
	@Override
	public DyBIPObject visitComponentType(DyBIPParser.ComponentTypeContext ctx) {
		String name = ctx.ID().getText();

		//----- ports
		int pSize = ctx.port().size();
		PortStmtList p = new PortStmtList();
		for(int i = 0; i <pSize; i++)			
			p.add(visit(ctx.port(i)).asPortStmtList());

		//----- data
		int dataSize = ctx.data().size();
		DataTypeStmtList data = new  DataTypeStmtList(dataSize);
		for(int i = 0; i < dataSize; i++)
			data.add(visit(ctx.data(i)).asDataTypeStmtList());

		//-----initTransition
		InitTransitionStmt initTrans = visit(ctx.initTransition()).asInitTransitionStmt();

		//-----transitions
		int transitionSize = ctx.transition().size();
		TransitionStmtList transitions = new TransitionStmtList(transitionSize);
		for(int i = 0 ; i < transitionSize; i++){
			TransitionStmt trans = visit(ctx.transition(i)).asTransitionStmt();
			transitions.add(trans);
		}
		ComponentTypeStmt ct = new ComponentTypeStmt(name, p, data, initTrans, transitions);
		
		return ct;
	}
	@Override
	public DyBIPObject visitTransition(DyBIPParser.TransitionContext ctx) {
		StateStmt fromState = new StateStmt(ctx.ID(1).getText());
		PortStmt port = new PortStmt(ctx.ID(0).getText());
		StateStmt toState = new StateStmt(ctx.ID(2).getText());
		GuardActionPair pair = visit(ctx.guardActionPair()).asGuardActionPair();
		
		TransitionStmt tr = new TransitionStmt(fromState, port, toState, pair);
		
		return tr;
	}
	@Override
	public DyBIPObject visitInitTransition(DyBIPParser.InitTransitionContext ctx) {
		StateStmt state = new StateStmt(ctx.ID().getText());
		GuardActionPair pair = visit(ctx.guardActionPair()).asGuardActionPair();
		
		InitTransitionStmt initT  = new InitTransitionStmt(state, pair);
		return initT;
	}
	@Override
	public DyBIPObject visitData(DyBIPParser.DataContext ctx) {
		int varNamesSize = ctx.ID().size();
		String type = ctx.type().getText();

		DataTypeStmtList data = new DataTypeStmtList(varNamesSize);
		for(int i = 0; i < varNamesSize; i++){
			data.add(new DataTypeStmt(type, ctx.ID(i).getText()));
		}
		return data;
	}
	@Override
	public DyBIPObject visitPort(DyBIPParser.PortContext ctx) {
		int size = ctx.ID().size();

		PortStmtList p = new PortStmtList(ctx.ID().size());
		for(int i = 0; i < size;i ++)
			p.add(new PortStmt(ctx.ID(i).getText()));

		return p;
	}
		
	/// -------------------------------------------------- Expression ----------------------
	@Override
	public DyBIPObject visitExpr(DyBIPParser.ExprContext ctx) {
		Expression expr = visit(ctx.getChild(0)).asExpression();
		return expr;
	} 
	@Override
	public DyBIPObject visitExprLiteral(DyBIPParser.ExprLiteralContext ctx) {
		Expression literal = null;
		
		String value = ctx.getText();
		
//		if(ctx.NEGATIVEINT() != null)
//			literal = new IntegerLiteral(value);
		if(ctx.INT() != null)
			literal = new IntegerLiteral(value);
		else if((ctx.TRUE() != null) || (ctx.FALSE() != null))
			literal = new BooleanLiteral(value);
		else if(ctx.NULL() != null)
			literal = new NullLiteral();
		
		return literal;
	}
	@Override
	public DyBIPObject visitExprVariable(DyBIPParser.ExprVariableContext ctx) {
		Expression expr = visit(ctx.getChild(0)).asExpression();
		return expr;
	}
	@Override
	public DyBIPObject visitVariable(DyBIPParser.VariableContext ctx) {
		Expression expr;
		int exprCount = ctx.expr().size();
		if(exprCount == 0){
			expr = new VariableExpression(ctx.ID().getText());
		}
		else if(exprCount == 1){
			// group access
			Expression index = visit(ctx.expr(0)).asExpression();
			expr = new GroupAccess(ctx.ID().getText(), index);
		}
		else {
			Expression beginIndex = visit(ctx.expr(0)).asExpression();
			Expression endIndex = visit(ctx.expr(1)).asExpression();
			expr = new SubGroupAccess(ctx.ID().getText(), beginIndex, endIndex);
		}
		return expr;
	}
	@Override
	public DyBIPObject visitDotAccess(DyBIPParser.DotAccessContext ctx) {
		int size = ctx.variable().size();
		VariableList vlist = new VariableList(size);
		for(int i = 0; i < size; i++){
			VariableExpression temp = visitVariable(ctx.variable(i)).asExprVariable();
			vlist.add(temp);
		}
		return vlist;
	}
	@Override
	public DyBIPObject visitSizeAccess(SizeAccessContext ctx) {
		int size = ctx.variable().size();
		SizeExpression vlist = new SizeExpression();
		for(int i = 0; i < size ; i++){ // skip last variable which is the size 
			VariableExpression temp = visitVariable(ctx.variable(i)).asExprVariable();
			vlist.add(temp);
		}
		return vlist;
	}
	@Override
	public DyBIPObject visitExprUnary(DyBIPParser.ExprUnaryContext ctx) {
		Expression expr= null;
		if(ctx.functionUnary() != null){
			expr = visit(ctx.functionUnary()).asExpression();
		}
		else{
			Expression e = visit(ctx.expr()).asExpression();
			expr = new ParenthesizedExpression(e);
		}
		return expr;
	}
	@Override
	public DyBIPObject visitFunctionUnary(DyBIPParser.FunctionUnaryContext ctx) {
		Expression e = visit(ctx.expr()).asExpression();
		String operator = ctx.getChild(0).getText();
		if(operator.equals("not"))
			return new Negation(e);
		return new UnaryFunction(operator, e);
	}
	@Override
	public DyBIPObject visitExprBinary(DyBIPParser.ExprBinaryContext ctx) {
		if(ctx.exprVariable() != null){
			Expression e1 = visit(ctx.exprVariable()).asExpression();
			Expression e2 = visit(ctx.expr()).asExpression();

			return new SetInclusion(e1, e2);
		}
		else{
			return visit(ctx.getChild(0));
		}
	}

	@Override
	public DyBIPObject visitExprQuadruple(ExprQuadrupleContext ctx) {
		Expression e1 = visit(ctx.expr(0)).asExpression();
		Expression e2 = visit(ctx.expr(1)).asExpression();
		Expression e3 = visit(ctx.expr(2)).asExpression();
		Expression e4 = visit(ctx.expr(3)).asExpression();
		
		return new ExprQuadruple(ctx.getChild(0).getText(), e1, e2, e3, e4);
	}
	
	@Override
	public DyBIPObject visitMathExpr(MathExprContext ctx) {
		Expression e1 = visit(ctx.getChild(0)).asExpression();
		String operator = ctx.moreMathTerms().op.getText(); // operator
		Expression e2 = visit(ctx.moreMathTerms().expr()).asExpression();

		return new MathmeticalOperation(e1, operator, e2);
	}
	@Override
	public DyBIPObject visitInequalityExpr(InequalityExprContext ctx) {
		Expression e1 = visit(ctx.getChild(0)).asExpression();
		String operator = ctx.moreEqualityTerms().op.getText();
		Expression e2 = visit(ctx.moreEqualityTerms().expr()).asExpression();
		
		return new Inequality(e1,operator, e2);
		
	}
	@Override
	public DyBIPObject visitEqualityExpr(EqualityExprContext ctx) {
		Expression e1 = visit(ctx.getChild(0)).asExpression();
		Expression e2 = visit(ctx.equalityTerms().expr()).asExpression();
		Expression expr = new Equality(e1, e2);
		return expr;
	}
	@Override
	public DyBIPObject visitAndorExpr(AndorExprContext ctx) {
		Expression e1 = visit(ctx.getChild(0)).asExpression();
		String operator = ctx.andorTerms().op.getText();
		Expression e2 = visit(ctx.andorTerms().expr()).asExpression();
		
		if(operator.equals("&"))	return new And(e1, e2);
		else return new Or(e1, e2);
	}
	@Override
	public DyBIPObject visitFunctionBinary(FunctionBinaryContext ctx) {
		String function = ctx.op.getText();
		Expression e1 = visit(ctx.expr(0)).asExpression();
		Expression e2 = visit(ctx.expr(1)).asExpression();

		return new BinaryFunction(e1, function, e2);
	}
	@Override
	public DyBIPObject visitExprAction(ExprActionContext ctx) {
		return visit(ctx.getChild(0));
	}
	@Override
	public DyBIPObject visitMotifAction(MotifActionContext ctx) {
		if(ctx.expr().size() == 2){
			Expression e1 = visit(ctx.expr(0)).asExpression();
			Expression e2 = visit(ctx.expr(1)).asExpression();
			
			String operator = ctx.getChild(0).getText();
			switch(operator){
			case "join":
				return new JoinAction(e1, e2);
			case "leave":
				return new LeaveAction(e1, e2);
			case "create":
				return new CreateAction(e1, e2);
			}
			return null;
		}
		else{
			Expression e1 = visit(ctx.expr(0)).asExpression();
			return new DeleteAction(e1);
		}
		
	}
	@Override
	public DyBIPObject visitAssignment(DyBIPParser.AssignmentContext ctx) {
		Expression e1 = visit(ctx.getChild(0)).asExpression();
		Expression e2 = visit(ctx.expr()).asExpression();

		Expression expr = new Assignment(e1, e2);
		return expr;
	}

	
	/// -------------------------------------------------- Guard Action Pair ----------------------
	@Override
	public DyBIPObject visitGuardActionPair(DyBIPParser.GuardActionPairContext ctx) {
		GuardList guard = visit(ctx.guard()).asGuardList();
		ActionList action = visit(ctx.action()).asActionList();	
		GuardActionPair pair = new GuardActionPair(guard, action);
		return pair;
	}
	@Override
	public DyBIPObject visitAction(DyBIPParser.ActionContext ctx) {
		int size = ctx.expr().size();
		ActionList action = new ActionList(size);
		for(int i = 0; i < size; i++){
			action.add(visit(ctx.expr(i)).asActionExpression());
		}
		return action;
	}
	@Override
	public DyBIPObject visitGuard(DyBIPParser.GuardContext ctx) {
		int size = ctx.expr().size();
		GuardList guard = new GuardList(size);
		for(int i = 0; i < size; i++){
			guard.add(visit(ctx.expr(i)).asBooleanExpression());
		}
		return guard;
	}
	
	// ----------------------------------------------------Motif TYpe -----------------------------
	@Override
	public DyBIPObject visitMotifType(MotifTypeContext ctx) {
		String typeName = ctx.ID().getText();
		
		int referSize = ctx.referMotif().size();
		ReferStmtList referList = new ReferStmtList(referSize);
		for(int i = 0; i < referSize; i++){
			referList.add(visit(ctx.referMotif(i)).asReferStmtList());
		}
		
		int instanceSize = ctx.componentInstance().size();
		ComponentInstanceStmtList instanceList = new ComponentInstanceStmtList(instanceSize);
		for(int i = 0; i < instanceSize; i++){
			ComponentInstanceStmtList instance = visit(ctx.componentInstance(i)).asComponentInstanceStmtList();
			instanceList.add(instance);
		}
		
		int rulesCount = ctx.rules().size();
		ReconfigurationRuleStmtList reconfigRuleList = new ReconfigurationRuleStmtList();
		InteractionRuleStmtList interactionRuleList = new InteractionRuleStmtList();	
		
		
		for(int i = 0; i< rulesCount; i++){
			DyBIPObject rule = visit(ctx.rules(i));
			if(rule instanceof ReconfigurationRuleStmt)
				reconfigRuleList.add(rule.asReconfigurationRuleStmt());
			else
				interactionRuleList.add(rule.asInteractionRuleStmt());
		}
		
		
		MotifTypeStmt motifType = new MotifTypeStmt(typeName, referList, instanceList, reconfigRuleList, interactionRuleList);
		return motifType;
		
	}
	@Override
	public DyBIPObject visitReferMotif(ReferMotifContext ctx) {
		int size = ctx.ID().size();
		String type = ctx.ID(0).getText();
		ReferStmtList referList= new ReferStmtList(size-1);
		for(int i = 1; i < size; i++){
			referList.add(new ReferStmt(type, ctx.ID(i).getText()));
		}
		return referList;
	}
	@Override
	public DyBIPObject visitComponentInstance(ComponentInstanceContext ctx) {
		int size = ctx.variable().size();
		ComponentInstanceStmtList instanceList = new ComponentInstanceStmtList(size);
		
		String type = ctx.ID().getText();
		
		for(int i = 0; i < size; i++){
			instanceList.add(new ComponentInstanceStmt(type, visit(ctx.variable(i)).asExprVariable()));
		}
		return instanceList;
	}

	// ----------------------------------------------------Motif Instance -----------------------------
	@Override
	public DyBIPObject visitMotifInstance(MotifInstanceContext ctx) {
		MotifInstanceStmtList motifInstanceList = new MotifInstanceStmtList();
		
		String type = ctx.ID(0).getText();
		for(int i = 1; i < ctx.ID().size(); i++){
			motifInstanceList.add(new MotifInstanceStmt(type, ctx.ID(i).getText()));
		}
		return motifInstanceList;
	}

	// ------------------------------------------------- Motif interaction ----------------
	@Override
	public DyBIPObject visitInteractionRule(DyBIPParser.InteractionRuleContext ctx) {
		 
		PortSymbolList portList = new PortSymbolList(ctx.ID().size());
		for(int i =0 ; i < ctx.ID().size(); i++){
			portList.add(ctx.ID(i).getText());
		}
		
		PortSymbolMap portMap = visit(ctx.portAssignStmts()).asPortSymbolMap();
		GuardActionPair guardAction = visit(ctx.guardActionPair()).asGuardActionPair();
		NestedIterators nestedIter = null;
		if(ctx.nestedIterators() != null){
			nestedIter= visit(ctx.nestedIterators()).asNestedIterators();
		}
		InteractionRuleStmt interaction = new InteractionRuleStmt(portList, portMap, guardAction, nestedIter);
		
		return interaction;
	}
	@Override
	public DyBIPObject visitPortAssignStmts(DyBIPParser.PortAssignStmtsContext ctx) {
		int size = ctx.portAssign().size();
		PortSymbolMap map = new PortSymbolMap(size);
		for(int i = 0; i < size; i++){
			String portSymbolName = ctx.portAssign(i).ID().getText();
			PortAccess portAccesss = visit(ctx.portAssign(i).portAccess()).asPortAccess();
			map.put(portSymbolName, portAccesss);
		}
		return map;
	}
	@Override
	public DyBIPObject visitPortAccess(PortAccessContext ctx) {
		String portName = ctx.ID().getText();
		PortAccess portAccess = new PortAccess(ctx.variable().size(), portName);
		for(int i =  0; i < ctx.variable().size(); i++){
			portAccess.add(visit(ctx.variable(i)).asExprVariable());
		}
		return portAccess;
	}
	
	
	// -------------------------------------------------- Iterators -------------- 
	@Override
	public DyBIPObject visitNestedIterators(DyBIPParser.NestedIteratorsContext ctx) {
		int iteratorCount = ctx.iterator().size();
		NestedIterators nestedIter = new NestedIterators(iteratorCount);
		for(int i = 0 ; i < iteratorCount ; i++)
			nestedIter.add(visit(ctx.iterator(i)).asIterator());

		return nestedIter;
	}
	@Override
	public DyBIPObject visitIterator(DyBIPParser.IteratorContext ctx) {
		return visit(ctx.getChild(0));
	}
	@Override
	public DyBIPObject visitForIterator(DyBIPParser.ForIteratorContext ctx) {
		String iteratorVar = ctx.ID().getText();

		Expression from = visit(ctx.range().expr(0)).asExpression();
		Expression to = visit(ctx.range().expr(1)).asExpression();
		Range range = new Range(from, to);
	

		if(ctx.expr() != null)
			return new ForIterator(iteratorVar, range, visit(ctx.expr()).asExpression());
		else
			return new ForIterator(iteratorVar, range);
	}
	
	@Override
	public DyBIPObject visitForEachIterator(ForEachIteratorContext ctx) {
		String iterartorType = ctx.ID(0).getText(); 
		String iteratorvar = ctx.ID(1).getText();
		
		if( ctx.expr() == null)
		 	return new ForEachIterator(iteratorvar, iterartorType);
		else
			return new ForEachIterator(iteratorvar, iterartorType, visit(ctx.expr()).asExpression()); 

	}
	
	@Override
	public DyBIPObject visitRules(DyBIPParser.RulesContext ctx) {
		return visit(ctx.getChild(0));
	}
	
	@Override
	public DyBIPObject visitReconfigurationRule(DyBIPParser.ReconfigurationRuleContext ctx) {
	
		NestedIterators nestedIter = visit(ctx.nestedIterators()).asNestedIterators();
		GuardActionPair guardAction =visit(ctx.guardActionPair()).asGuardActionPair();
		return new ReconfigurationRuleStmt(nestedIter, guardAction);
	}
	
	@Override
	public DyBIPObject visitInterReconfigurationRule(DyBIPParser.InterReconfigurationRuleContext ctx) {
		NestedIterators nestedIter = visit(ctx.nestedIterators()).asNestedIterators();
		GuardActionPair guardAction =visit(ctx.guardActionPair()).asGuardActionPair();
		
		return new InterReconfigurationRuleStmt(nestedIter, guardAction);
	}
	
	

}
