package dybip.InteractionStmt;

import java.util.Arrays;

import dybip.Context;
import dybip.IndexMap;
import dybip.expression.booleanExpr.BooleanExpression;
import dybip.expression.list.ActionList;
import dybip.expression.list.ExpressionList;
import dybip.expression.list.GuardList;
import dybip.expression.literal.BooleanLiteral;
import dybip.system.SystemContext;

public class ExecutableInterReconfigurationRule extends ExecutableRule{

	private int [] start;
	private int [] end;
	private int [] counters;
	private String [] counterVarNames;
	private String [] typeStr;
	private ExpressionList<BooleanExpression> conditionList;
	private ActionList actionList;
	private GuardList guardList;

	private EnabledStepList<EnabledInterReconfiguration> enabledMoveList;
	public ExecutableInterReconfigurationRule(RuleStmt stmt, Context context){
		super(stmt, context);

		InterReconfigurationRuleStmt rule = (InterReconfigurationRuleStmt) stmt;
		if(rule.getNestedIterators() != null) {
			NestedIterators iter = rule.getNestedIterators();

			start = new int[iter.size()];
			end = new int[iter.size()];
			counters  =  new int [iter.size()];
			counterVarNames = new String [iter.size()];
			typeStr = new String [iter.size()];
			conditionList = new ExpressionList<BooleanExpression>(iter.size());

			for(int i = 0; i < iter.size(); i++){
				// create range from type 

				Iterator iterator = iter.get(i);
				if(iterator instanceof ForEachIterator) {
					String  type = ((ForEachIterator) iterator).getIteratorType();
					SystemContext systemContext = (SystemContext) context;
					
					start[i] = 0;
					end[i] = systemContext.getMotifListOfType(type).size();
					typeStr[i] = type;	
					counters[i] = 0;
					counterVarNames[i] = iterator.getIteratorVar();
					if(iter.get(i).getExpr() == null)
						conditionList.add(new BooleanLiteral(true));
					else
						conditionList.add((BooleanExpression) iterator.getExpr());
				}
			}
			guardList = rule.getGuards();
			actionList = rule.getActions();
		}
		
	}
	public EnabledStepList<EnabledInterReconfiguration> getEnabledSteps(){
		return enabledMoveList;
	}
	public void updateEndIndexes(Context context) {
		SystemContext systemContext = (SystemContext) context;

		for(int i = 0; i < end.length; i++){
			start[i] = 0;
			end[i] = systemContext.getMotifListOfType(typeStr[i]).size()  - 1 ;
		}
	}
	public EnabledStepList<EnabledInterReconfiguration> evaluateEnabledInterReconfiguration(Context context){
		enabledMoveList = new EnabledStepList<EnabledInterReconfiguration>();
		updateEndIndexes(context);
		nestedLoopOperation(0, counters, counterVarNames, start, end, conditionList, guardList, actionList, context);
		return enabledMoveList;
	}
	public void nestedLoopOperation(int level, int [] counters, String [] countersStr, int [] start, int [] end, ExpressionList<BooleanExpression> condition, GuardList guards, ActionList updateExpr, Context context) {
		if(level == counters.length){ // we want to perform a loop at last level 
			if(guards.minify(context).value(Boolean.class)){ // interaction guards enabled
				IndexMap indexMap = new IndexMap();
				for(int i = 0; i < countersStr.length;i++)
					indexMap.put(countersStr[i], counters[i]);
				
				EnabledInterReconfiguration step = new EnabledInterReconfiguration(updateExpr, indexMap);
				if(!enabledMoveList.contains(step))
					enabledMoveList.add(step);
			}
		}
		else{
			// ASSUMPTION component indexing always starts from zero 
			for(counters[level] = start[level], context.putIndex(countersStr[level], counters[level]), context.putMotifList(counterVarNames[level], ((SystemContext)context).getMotifListOfType(typeStr[level]));
						counters[level] <= end[level] ;
							counters[level]++, context.putIndex(countersStr[level], counters[level]), context.putMotifList(counterVarNames[level], ((SystemContext)context).getMotifListOfType(typeStr[level]))){	
				if(condition.get(level).minify(context).value(Boolean.class))
					nestedLoopOperation(level+1, counters, countersStr, start, end, condition, guards, updateExpr, context);		
				}
		}
	}
	public boolean hasEnabledStep() {
		return enabledMoveList.size() != 0;
	}
	private String intArrayToString(int [] array){
		String[] a = Arrays.toString(array).split("[\\[\\]]")[1].split(", "); 
		return stringArrayToString(a);
	}
	private String stringArrayToString(String [] array){
		String str = "";
		for(int i = 0; i< array.length; i++)
			if(i == array.length-1)
				str += array[array.length -1] + "\n";
			else	
				str += array[i] + "\t";
		return str;
	}
	@Override
	public String toString(int toStringMode){
		String str = "";
		if(start.length != 0)
			str += "\n\t\t\t\tStarting Counters:\t" + intArrayToString(start)
			+ "\t\t\t\tEnd Counters:\t\t" + intArrayToString (end)
			+ "\t\t\t\tCurrent Counters:\t" + intArrayToString(counters)
			+ "\t\t\t\tCounter Variables:\t" + stringArrayToString(counterVarNames)
			+ "\t\t\t\tType Variables:\t" + stringArrayToString(typeStr)
			+ "\t\t\t\tCondition Expression:\t" + conditionList.toString() + "\n"; 

		str += "\t\t\t\tGuard:\t\t\t" + guardList.toString().replace("[", "").replace("]", "") + "\n"
				+ "\t\t\t\tAction:\t\t\t" + actionList.toString().replace("{", "").replace("}", "") + "\n";
		return str;
	}
}
