package dybip.InteractionStmt;

import java.util.Arrays;

import dybip.Context;
import dybip.IndexMap;
import dybip.componentInstance.ComponentInstance;
import dybip.expression.Expression;
import dybip.expression.booleanExpr.BooleanExpression;
import dybip.expression.list.ActionList;
import dybip.expression.list.ExpressionList;
import dybip.expression.list.GuardList;
import dybip.expression.list.PortReferenceList;
import dybip.expression.literal.BooleanLiteral;
import dybip.expression.variable.PortAccess;
import dybip.expression.variable.VariableExpression;
import dybip.expression.variable.VariableList;
import dybip.motifInstance.Group;

public class ExecutableInteractionRule extends ExecutableRule{
	private int [] start;
	private int [] end;
	private int [] counters;
	private String [] counterVarNames;
	private ExpressionList<BooleanExpression> conditionList;
	private ActionList actionList;
	private GuardList guardList;
	private PortReferenceList portList;
	
	private Expression [] startIndexExpr;
	private Expression [] endIndexExpr;
	
	private EnabledStepList<EnabledInteraction> enabledMoveList; //not sure this is wise!
	public ExecutableInteractionRule(RuleStmt stmt, Context context){
		super(stmt, context);
		InteractionRuleStmt portInter = (InteractionRuleStmt) stmt;
		if(portInter.getNestedIterators() != null){
			NestedIterators iter = portInter.getNestedIterators();

			start = new int[iter.size()];
			end = new int[iter.size()];
			startIndexExpr = new Expression[iter.size()];
			endIndexExpr = new Expression[iter.size()];
			counters  =  new int [iter.size()];
			counterVarNames = new String [iter.size()];
			conditionList = new ExpressionList<BooleanExpression>(iter.size());
			for(int i = 0; i < iter.size(); i++){
				Range r = ((ForIterator)iter.get(i)).getRange();
				startIndexExpr[i] = r.getFrom();
				endIndexExpr[i] = r.getTo();			
				
				start[i] = startIndexExpr[i].minify(context).value(Integer.class);
				end[i] = endIndexExpr[i].minify(context).value(Integer.class);
				counters[i] = 0;
				counterVarNames[i] = iter.get(i).getIteratorVar();
				if(iter.get(i).getExpr() == null)
					conditionList.add(new BooleanLiteral(true));
				else
					conditionList.add((BooleanExpression) iter.get(i).getExpr());

			}
			guardList = portInter.getGuards();
			portList = portInter.getPortList();
			actionList = portInter.getActions();
		}
		else{
			start = new int[0];
			end = new int [0];
			counters = new int[0];
			counterVarNames = new String[0];
			conditionList = null;
			guardList = portInter.getGuards();
			portList = portInter.getPortList();
			actionList = portInter.getActions();
		}
	}
	public void updateBeginEndIndexes(Context context) {
		for(int i = 0; i < startIndexExpr.length; i++){
			start[i] = startIndexExpr[i].minify(context).value(Integer.class);
			end[i] = endIndexExpr[i].minify(context).value(Integer.class);
		}
	}

	public EnabledStepList<EnabledInteraction> evaluateEnabledInteraction(Context context){
		enabledMoveList = new EnabledStepList<EnabledInteraction>();
		if(start.length != 0 && end.length != 0) // there is a nested for loop in interaction update the begin and start index
			updateBeginEndIndexes(context);
		nestedLoopOperation(0, counters, counterVarNames, start, end, conditionList, guardList, actionList, portList, context);
		return enabledMoveList;
	}

	// TODO when completely done remove all indexes added after execution of some interaction
	public void nestedLoopOperation(int level, int [] counters, String [] countersStr, int [] start, int [] end, ExpressionList<BooleanExpression> condition, GuardList guards, ActionList updateExpr, PortReferenceList portList, Context context){

		if(level == counters.length){ // we want to perform a loop at last level 
			if(guards.minify(context).value(Boolean.class) && portList.minify(context).value(Boolean.class)){ // interaction guards enabled

				
				boolean enabledInteraction = true;
				for(int i = 0; i < portList.size() & enabledInteraction; i++)
				{
					Group compArray  = (Group) (portList.get(i).getVariableList()).minify(context);
					for(int j = 0; j < compArray.size(); j++){
						ComponentInstance citemp = compArray.get(j);
						// update data map
						context.setDataMap(citemp.getDataMap());
						GuardList transitionGuard = citemp.getCtPointer().getTransitionGuard(citemp.getCurrentStatePointer(), portList.get(i).getPortName());
						enabledInteraction &= transitionGuard.minify(context).value(Boolean.class);	
						context.resetDataMap();
					}
					
				}
				if(enabledInteraction) {
					IndexMap indexMap = new IndexMap();
					for(int i = 0; i < countersStr.length;i++)
						indexMap.put(countersStr[i], counters[i]);

					EnabledInteraction interaction = new EnabledInteraction(updateExpr, indexMap);
					addInteraction(interaction, portList, updateExpr, context);
					if(interaction.size() > 0 && !enabledMoveList.contains(interaction))
						enabledMoveList.add(interaction);
				}
//				GuardList transitionGuards = new GuardList();
//				for(int i = 0; i < portList.size(); i++)
//				{
//					Group compArray  = (Group) (portList.get(i).getVariableList()).minify(context);
//					for(int j = 0; j < compArray.size(); j++){
//						ComponentInstance citemp = compArray.get(j);
//						transitionGuards.addAll(citemp.getCtPointer().getTransitionGuard(citemp.getCurrentStatePointer(), portList.get(i).getPortName()));}
//				}
//					
//				if(transitionGuards.minify(context).value(Boolean.class)){ // port enabled & transition guards enabled
//					IndexMap indexMap = new IndexMap();
//					for(int i = 0; i < countersStr.length;i++)
//						indexMap.put(countersStr[i], counters[i]);
//
//					EnabledInteraction interaction = new EnabledInteraction(updateExpr, indexMap);
//					addInteraction(interaction, portList, updateExpr, context);
//					if(interaction.size() > 0 && !enabledMoveList.contains(interaction))
//						enabledMoveList.add(interaction);
//				}
			}

		}
		else{
			// ASSUMPTION component indexing always starts from zero 
			for(counters[level] = start[level], context.putIndex(countersStr[level], counters[level]); counters[level] <= end[level] ; counters[level]++, context.putIndex(countersStr[level], counters[level]) ){	
				if(condition.get(level).minify(context).value(Boolean.class))
					nestedLoopOperation(level+1, counters, countersStr, start, end, condition, guards, updateExpr, portList, context);}		
		}
	}

	public void addInteraction(EnabledInteraction interaction, PortReferenceList portList, ActionList updateExpr, Context context){
		//		 add all paretcipating 

		for(int i = 0; i < portList.size(); i++){
			
			PortAccess portRef = portList.get(i);
			String portName = portRef.getPortName();
			
			Group interactingGroup = getInteractingGroup(portRef, context);
			
			Expression exprVar = portRef.getVariableList().minify(context);
			if(exprVar instanceof Group){
				Group cai = (Group) exprVar;
				for(int j = 0; j < cai.size(); j++){
					ComponentInstance instance = cai.get(j);
					InteractingComponent ic = new InteractingComponent(instance, interactingGroup, portName);
					interaction.add(ic);
				}
			}

			else
			{// ERROR: referncing  a port from a noncomponent variable / non existing variable 
				;
			}
		}

	}
	private Group getInteractingGroup(PortAccess portRef, Context context) {
		VariableList varExprList = portRef.getVariableList();
		if(varExprList.size() == 1) { // groupName.port || grouPName[i].port || groupName[i:j].port
			VariableExpression varExpr = varExprList.get(0);
			String varName = varExpr.getVariableName();
			Expression expr = context.find(varName);
			if(expr instanceof Group) {
				return (Group)expr;
			}
			else {
				// nonexisting var
				return null;
			}
		}
		else { // motifName.groupName.port || motifName.groupName[i].port || motifName.groupName[i:j].port
			return null;
		}
	}
	@Override
	public String toString(int toStringMode){
		String str = "";
		if(start.length != 0)
			str += "\n\t\t\t\tStarting Counters:\t" + intArrayToString(start)
			+ "\t\t\t\tEnd Counters:\t\t" + intArrayToString (end)
			+ "\t\t\t\tCurrent Counters:\t" + intArrayToString(counters)
			+ "\t\t\t\tCounter Variables:\t" + stringArrayToString(counterVarNames)
			+ "\t\t\t\tCondition Expression:\t" + conditionList.toString() + "\n"; 

		str += "\t\t\t\tGuard:\t\t\t" + guardList.toString().replace("[", "").replace("]", "") + "\n"
				+ "\t\t\t\tAction:\t\t\t" + actionList.toString().replace("{", "").replace("}", "") + "\n"
				+ "\t\t\t\tPorts:\t\t\t" + portList.toString();
		return str;
	}
	private String intArrayToString(int [] array){
		String[] a = Arrays.toString(array).split("[\\[\\]]")[1].split(", "); 
		return stringArrayToString(a);
	}
	private String stringArrayToString(String [] array){
		String str = "";
		for(int i = 0; i< array.length; i++)
			if(i == array.length-1)
				str += array[array.length -1] + "\n";
			else	
				str += array[i] + "\t";
		return str;
	}
}
