package dybip.InteractionStmt;

import java.util.Arrays;

import dybip.Context;
import dybip.IndexMap;
import dybip.expression.Expression;
import dybip.expression.booleanExpr.BooleanExpression;
import dybip.expression.list.ActionList;
import dybip.expression.list.ExpressionList;
import dybip.expression.list.GuardList;
import dybip.expression.literal.BooleanLiteral;


public class ExecutableReconfigurationRule extends ExecutableRule{

	private int [] start;
	private int [] end;
	private int [] counters;
	private String [] counterVarNames;
	private ExpressionList<BooleanExpression> conditionList;
	private ActionList actionList;
	private GuardList guardList;

	private Expression [] startIndexExpr; // store expression as need to re evaluate them after each iteration
	private Expression [] endIndexExpr;
	
	private EnabledStepList<EnabledReconfiguration> enabledMoveList;
	public ExecutableReconfigurationRule(RuleStmt stmt, Context context){
		super(stmt, context);
		ReconfigurationRuleStmt rule = (ReconfigurationRuleStmt) stmt;
		if(rule.getNestedIterators() != null){
			NestedIterators iter = rule.getNestedIterators();

			start = new int[iter.size()];
			end = new int[iter.size()];
			startIndexExpr = new Expression[iter.size()];
			endIndexExpr = new Expression[iter.size()];
			counters  =  new int [iter.size()];
			counterVarNames = new String [iter.size()];
			conditionList = new ExpressionList<BooleanExpression>(iter.size());
			
			for(int i = 0; i < iter.size(); i++){
				Iterator iterator = iter.get(i);
				Range r = ((ForIterator)iterator).getRange();
				
				startIndexExpr[i] = r.getFrom();
				endIndexExpr[i] = r.getTo();			
				
				start[i] = startIndexExpr[i].minify(context).value(Integer.class);
				end[i] = endIndexExpr[i].minify(context).value(Integer.class);
				counters[i] = 0;
				counterVarNames[i] = iterator.getIteratorVar();
				if(iter.get(i).getExpr() == null)
					conditionList.add(new BooleanLiteral(true));
				else
					conditionList.add((BooleanExpression) iterator.getExpr());

			}
			guardList = rule.getGuards();
			actionList = rule.getActions();
		}

	}
	public void updateBeginEndIndexes(Context context) {
		for(int i = 0; i < startIndexExpr.length; i++){
			start[i] = startIndexExpr[i].minify(context).value(Integer.class);
			end[i] = endIndexExpr[i].minify(context).value(Integer.class);
		}
	}
	public EnabledStepList<EnabledReconfiguration> evaluateEnabledReconfiguration(Context context){
		enabledMoveList = new EnabledStepList<EnabledReconfiguration>();
		updateBeginEndIndexes(context);
		nestedLoopOperation(0, counters, counterVarNames, start, end, conditionList, guardList, actionList, context);
		return enabledMoveList;
	}
	// TODO when completely done remove all indexes added after execution of some interaction
	public void nestedLoopOperation(int level, int [] counters, String [] countersStr, int [] start, int [] end, ExpressionList<BooleanExpression> condition, GuardList guards, ActionList updateExpr, Context context){

		if(level == counters.length){ // we want to perform a loop at last level 
			if(guards.minify(context).value(Boolean.class)){ // interaction guards enabled
				IndexMap indexMap = new IndexMap();
				for(int i = 0; i < countersStr.length;i++)
					indexMap.put(countersStr[i], counters[i]);
				
				EnabledReconfiguration step = new EnabledReconfiguration(updateExpr, indexMap);
				if(!enabledMoveList.contains(step))
					enabledMoveList.add(step);
			}
		}
		else{
			// ASSUMPTION component indexing always starts from zero 
			for(counters[level] = start[level], context.putIndex(countersStr[level], counters[level]); 
						counters[level] <= end[level] ; 
							counters[level]++, context.putIndex(countersStr[level], counters[level]) ){	
				if(condition.get(level).minify(context).value(Boolean.class))
					nestedLoopOperation(level+1, counters, countersStr, start, end, condition, guards, updateExpr, context);}		
		}
	}
	private String intArrayToString(int [] array){
		String[] a = Arrays.toString(array).split("[\\[\\]]")[1].split(", "); 
		return stringArrayToString(a);
	}
	private String stringArrayToString(String [] array){
		String str = "";
		for(int i = 0; i< array.length; i++)
			if(i == array.length-1)
				str += array[array.length -1] + "\n";
			else	
				str += array[i] + "\t";
		return str;
	}
	@Override
	public String toString(int toStringMode){
		String str = "";
		if(start.length != 0)
			str += "\n\t\t\t\tStarting Counters:\t" + intArrayToString(start)
			+ "\t\t\t\tEnd Counters:\t\t" + intArrayToString (end)
			+ "\t\t\t\tCurrent Counters:\t" + intArrayToString(counters)
			+ "\t\t\t\tCounter Variables:\t" + stringArrayToString(counterVarNames)
			+ "\t\t\t\tCondition Expression:\t" + conditionList.toString() + "\n"; 

		str += "\t\t\t\tGuard:\t\t\t" + guardList.toString().replace("[", "").replace("]", "") + "\n"
				+ "\t\t\t\tAction:\t\t\t" + actionList.toString().replace("{", "").replace("}", "") + "\n";
		return str;
	}
}
