package dybip.componentInstance;

import java.util.Arrays;
import java.util.List;

import dybip.DyBIPRunner;
import dybip.GroupList;
import dybip.componentType.ComponentType;
import dybip.componentType.State;
import dybip.expression.Expression;
import dybip.expression.list.ActionList;
import dybip.expression.list.GuardList;
import dybip.expression.literal.BooleanLiteral;
import dybip.motifInstance.Group;
import dybip.motifInstance.MotifContext;
import dybip.Context;

public class ComponentInstance  extends Expression{ // extends expression as it represents a variable name
	private ComponentType ctPointer; //pointer to component type
	private State currentStatePointer; // current state 
	private DataMap dataMap; // map of component data and their values
	
	private GroupList groupPtrList; // pointer to group
	
	private static long counter = 0;
	private final long id;
	public ComponentInstance(Group group,ComponentType ct, Context context){
		groupPtrList = new GroupList();
		groupPtrList.add(group);
		ctPointer = ct;
		initDataMap();
		execInitialTransition(context);
		id = counter++;
	}
	public ComponentInstance(ComponentType ct, Context context) {
		groupPtrList = new GroupList();
		ctPointer = ct;
		initDataMap();
		execInitialTransition(context);
		id = counter++;
	}
	private void initDataMap(){
		// give data default values
		dybip.componentType.DataTypeList dataList = ctPointer.getData();
		dataMap = new DataMap();
		for(int i = 0; i < dataList.size(); i++)
		{
			dybip.componentType.DataType d = dataList.get(i);
			String type = d.getType();
		
				String name = d.getName();
				Data dd = null;
				switch (type){
				case "int":
					dd = new IntegerData(name);
					break;
				case "boolean":
					dd = new BooleanData(name);
					break;
				}
				dataMap.put(name, dd);
			
		}
	}
	
	private void execInitialTransition(Context context){
		GuardList guard = ctPointer.getInitialGuard();
		ActionList action = ctPointer.getInitialAction();
		
		if(guard.minify(context).value(Boolean.class)){
			context.setDataMap(dataMap); // add component instance variables to context
			action.minify(context);
			context.resetDataMap();
		}
		currentStatePointer = ctPointer.getStartState();
	}
	private void execInternalTransition(Context context) {
		
	}
	public long getID() {
		return id;
	}
	public ComponentType getCtPointer() {
		return ctPointer;
	}
	public State getCurrentStatePointer() {
		return currentStatePointer;
	}
	public DataMap getDataMap() {
		return dataMap;
	}
	public GroupList getGroupPtrList(){
		return groupPtrList;
	}
	public void resetGroupPtrList() {
		groupPtrList = new GroupList();
	}
	
	public void removeGroup(Group group) {
		groupPtrList.remove(group);
	}
	public void addGroup(Group group) {
		groupPtrList.add(group);
	}
	
	public void setData(String name, Expression data){
		((dataMap.get(name))).setData(data);
	}
	public Class<?> getInstanceType(Context context){
		return ctPointer.getClassType(context);
	}
	public List<String> getEnabledPort(){
		return Arrays.asList(ctPointer.getEnabledPort(currentStatePointer));
	}
	
	@Override
	public Expression minify(Context context) {
		return this;
	}
	@Override
	public Class<?> getExpressionType(Context context) {
		return ComponentInstance.class;
	}
	
	
	public boolean makeTransition(String p, Context context){
		if(context instanceof MotifContext){
			MotifContext mContext = (MotifContext) context;
			State curr = getCurrentStatePointer();
			if(ctPointer.transitionExists(curr, p)){
				mContext.setDataMap(dataMap); // add component instance variables to context
				ActionList actionList = ctPointer.getTransitionAction(curr, p);
				BooleanLiteral completed = (BooleanLiteral) actionList.minify(context);
				mContext.resetDataMap();
				currentStatePointer = ctPointer.getNextState(curr, p);
				return completed.value(Boolean.class);}
		}
		return false;	
	}
	public Data getData(String varName) {
		return dataMap.get(varName);
	}
	
	public String getTrace() {
		return dataMap.getTrace().replaceAll("REPLACE2", ctPointer.getName() + id);
	}
	
	@Override
	public String toString(){
		return "{" + ctPointer.getName() + " " + currentStatePointer + " "  +  dataMap.toString(DyBIPRunner.NORMAL) +  " " + groupPtrList.toString() + " id "+ id + "}";
	}
	@Override
	public String toMinifyString(Context context){
		return "";
	}
	@Override
	public String toString(int toStringMode){
		String str = "{";
		if(toStringMode == DyBIPRunner.DEBUG)
			str += ctPointer.getName() + " "  + currentStatePointer.getName() + " " + dataMap.toString(toStringMode) + " " + groupPtrList.toString() + " " + id;
		else if(toStringMode == DyBIPRunner.NORMAL)
			str += currentStatePointer.getName() + " " + dataMap.toString(toStringMode) + " " + groupPtrList.toString() + " " + id;

		return str + "}";
	}
	
	
}

//public ComponentInstance(ComponentTypeMap ctMap, ComponentInstanceStatement stmt){
//ctPointer = ctMap.get(stmt.getType());
//currentStatePointer = ctPointer.getStartState();
//initDataMap();
//}

//else return false; // ERROR no transition on port p from given state
//}
//public boolean makeInitialTransition(Context context){
//// ASSUMPTION current state is null if guard is false
//if(ctPointer.getInitialGuard().minify(context).value(Boolean.class)){
//	context.setDataMap(dataMap); // add component instance variables to context
//	currentStatePointer = ctPointer.getStartState();
//	BooleanLiteral completed = (BooleanLiteral) ctPointer.getInitialAction().minify(context);
//	context.resetDataMap();
//	return completed.value(Boolean.class);
//}
//return false; // ERROR no 
//}