package dybip.componentType;

import java.util.Arrays;
import dybip.expression.Expression;
import dybip.expression.list.ActionList;
import dybip.expression.list.GuardList;
import dybip.Context;

/**
 * This class represents a Component Type in a Dynamic BIP model
 * @author el-ballouli
 *
 */
public class ComponentType extends Expression{
	private String name;
	private DataTypeList data;
	private InitTransition initTrans;
	private TransitionMap transitionMap;
	
	public Class<?> getClassType(Context context){
		try {
			return Class.forName("dybip.componentType.generated." + name);
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return ClassNotFoundException.class;
		}
	}
	
	/**
	 * Constructs a new component type
	 * @param name
	 * @param p
	 * @param d
	 * @param initTrans
	 * @param transition
	 */
	public ComponentType(String name, DataTypeList d, InitTransition initTrans, TransitionMap transition){
		this.name = name;
		this.data = d;
		this.initTrans = initTrans;
		this.transitionMap = transition;
	}
	public TransitionMap getTransitionMap(){
		return transitionMap;
	}
	public State getNextState(State cur, String p){
		return (transitionMap.getTransition(cur, p)).getToState();
	}
	public GuardList getTransitionGuard(State state, String port){
		return transitionMap.getTransitionGuard(state, port);		
	}
	public ActionList getTransitionAction(State state, String port){
		return transitionMap.getTransitionAction(state, port);		
	}
	/**
	 * 
	 * @return return name of component type
	 */
	public String getName(){
		return name;
	}
	public State getStartState(){
		return initTrans.getStartState();
	}
	public GuardList getInitialGuard(){
		return initTrans.getGuardList();
	}
	public ActionList getInitialAction(){
		return initTrans.getActionList();
	}
	/**
	 * 
	 * @return returns list of data in the component type
	 */
	public DataTypeList getData(){
		return data;
	}
	/**
	 * 
	 * @param currentState
	 * @return return a list of port names that are enabled from a given state 
	 */
	public String [] getEnabledPort(State currentState){
		TransitionList trans = transitionMap.getTransitionList(currentState);
		String [] enabledPort = new String [trans.size()];
		for(int i = 0; i < trans.size(); i++)
			enabledPort[i] = trans.get(i).getPort().getName();
		return enabledPort;
	}
	public boolean transitionExists(State state, String port){
		return Arrays.asList(getEnabledPort(state)).contains(port);
		
	}
	@Override
	public String toString(int toStringMode){
		String result = "";
		return result;
	}
	@Override
	public String toString()
	{
		return  name + " " + data.toString() + "\n";
	}

	@Override
	public Expression minify(Context context) {
		return this;
	}

	@Override
	public Class<?> getExpressionType(Context context) {
		return ComponentType.class;
	}

	@Override
	public String toMinifyString(Context context) {
		return name;
	}
}
