package dybip.componentType;

import dybip.DyBIPHashMap;
import dybip.expression.list.ActionList;
import dybip.expression.list.GuardList;
import dybip.systemStmt.componentTypeStmt.TransitionStmt;
import dybip.systemStmt.componentTypeStmt.TransitionStmtList;

/**
 * Class representing a map of ports and their corresponding transition 
 * @author el-ballouli
 *
 */
class PortTransitionMap extends DyBIPHashMap<String, Transition>{
	
	public Port getPort(String str){
		return get(str).getPort();
	}
	@Override
	public String toString(){
		String str = "";
		for(String key : keySet())
			str += "\t\t" + get(key).toString()+"\n";
		return str;
	}
}
/**
 * This class represents a map of start states and all transitions allowed from that start state
 * @author el-ballouli
 *
 */
public class TransitionMap extends DyBIPHashMap<String, PortTransitionMap>{
	
	public TransitionMap(int size){
		super(size);
	}
	public TransitionMap(){
		super();
	}
	public TransitionMap(TransitionStmtList tl){
		super();
		for(int i = 0; i < tl.size(); i++){
			put(tl.get(i));
		}
	}
	public void put(TransitionStmt t){
		put(t.getFromState().getName(), new Transition(t));
	}
	/**
	 * Add a state and transition that can be done from this state to the map. if there exists a transition from this state in the map 
	 * it gets added to the map of transitions that can be done from this state other wise a new pair is added to th emap 
	 * @param startState
	 * @param t
	 */
	public void put(String startState, Transition t){
		if(containsKey(startState))
			get(startState).put(t.getPort().getName(), t);
		else{
			PortTransitionMap portStateMap = new PortTransitionMap();
			portStateMap.put(t.getPort().getName(), t);
			put(startState, portStateMap);
		}
	}
	/**
	 * 
	 * @param s
	 * @return return all transitions possible from a given state
	 */
	public TransitionList getTransitionList(State s){
		TransitionList transitonList = new TransitionList();
		PortTransitionMap mp = get(s.getName());
		for(String port: mp.keySet())
			transitonList.add(mp.get(port));
		
		return transitonList;
	}
	/**
	 * 
	 * @param st
	 * @param p
	 * @return the transition from a given state on a given port
	 */
	public Transition getTransition(State st, String p){
		return get(st.getName()).get(p);
	}
	public GuardList getTransitionGuard(State st, String p){
		return getTransition(st, p).getGuard();
	}
	public ActionList getTransitionAction(State st, String p){
		return getTransition(st, p).getAction();
	}
	@Override
	public String toString(){
		String result = "";
		for(String key : keySet()){
			result +=  get(key).toString() ;}
		return result;
	}	
}
