package dybip.expression;

import dybip.Context;
import dybip.expression.list.DataList;
import dybip.expression.literal.IntegerLiteral;

public class ExprQuadruple extends Expression{
	Expression expr1;
	Expression expr2;
	
	Expression expr3;
	Expression expr4;
	
	String operator;
	public ExprQuadruple(String op, Expression e1, Expression e2, Expression e3, Expression e4) {
		this.expr1 = e1;
		this.expr2 = e2;
		this.expr3 = e3;
		this.expr4 = e4;
		this.operator = op;
	}

	@Override
	public Expression minify(Context context) {
		if (operator.equals("dist"))
			return minifyDistance(context);
		else if(operator.equals("angle"))
			return minifyAngle(context);
		else return null;
	}
	private Expression minifyDistance(Context context) {
		Expression e1 = expr1.minify(context);
		Expression e2 = expr2.minify(context);
		Expression e3 = expr3.minify(context);
		Expression e4 = expr4.minify(context);
		
		Class<?> c1 = e1.getExpressionType(context);
		Class<?> c2 = e2.getExpressionType(context);
		Class<?> c3 = e3.getExpressionType(context);
		Class<?> c4 = e4.getExpressionType(context);
		
		if(c1 == DataList.class && c2 == DataList.class
				& c3 == DataList.class && c4 == DataList.class)
		{
			DataList list1 = (DataList)e1;
			DataList list2 = (DataList)e2;
			DataList list3 = (DataList)e3;
			DataList list4 = (DataList)e4;
			if(list1.getListType(context) == Integer.class && list2.getListType(context) == Integer.class &&
					list3.getListType(context) == Integer.class && list4.getListType(context) == Integer.class &&
					list1.size() == 1 && list2.size() == 1 && list3.size() == 1 && list4.size() == 1 )
			{
				Integer i1 = list1.get(0).minify(context).value(Integer.class);
				Integer i2 = list2.get(0).minify(context).value(Integer.class); 
			
				Integer i3 = list3.get(0).minify(context).value(Integer.class);
				Integer i4 = list4.get(0).minify(context).value(Integer.class); 

				Integer xDiff = (int) Math.pow(i1-i3,2);
				Integer yDiff = (int) Math.pow(i2-i4,2);
				
				return new IntegerLiteral((int) Math.sqrt(xDiff+yDiff));			
			}
		}
		else if(c1 == Integer.class && c2 == Integer.class &&
				c3 == Integer.class && c4 == Integer.class)
		{
			Integer i1 = e1.value(Integer.class);
			Integer i2 = e2.value(Integer.class); 
		
			Integer i3 = e3.value(Integer.class);
			Integer i4 = e4.value(Integer.class); 

			Integer xDiff = (int) Math.pow(i1-i3,2);
			Integer yDiff = (int) Math.pow(i2-i4,2);
			
			return new IntegerLiteral((int) Math.sqrt(xDiff+yDiff));	
		}
		System.err.println("error here");
		return null;
	}
	private Expression minifyAngle(Context context) {
		Expression e1 = expr1.minify(context);
		Expression e2 = expr2.minify(context);
		Expression e3 = expr3.minify(context);
		Expression e4 = expr4.minify(context);
		
		Class<?> c1 = e1.getExpressionType(context);
		Class<?> c2 = e2.getExpressionType(context);
		Class<?> c3 = e3.getExpressionType(context);
		Class<?> c4 = e4.getExpressionType(context);
		
		if(c1 == Integer.class && c2 == Integer.class &&
				c3 == Integer.class && c4 == Integer.class){
			Integer i1 = e1.value(Integer.class);
			Integer i2 = e2.value(Integer.class); 
		
			Integer i3 = e3.value(Integer.class);
			Integer i4 = e4.value(Integer.class); 

			Integer xDiff = (int) i3-i1;
			Integer yDiff = (int) i4-i2;
			
//			(Math.atan2(xDiff, yDiff)*180)/ Math.PI;
			return new IntegerLiteral(Math.atan2(yDiff, xDiff) * 180 / Math.PI);	
		}
		System.err.println("error here");
		return null;
	}
	@Override
	public Class<?> getExpressionType(Context context) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String toString() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String toMinifyString(Context context) {
		// TODO Auto-generated method stub
		return null;
	}
}
