package dybip.expression.action;

import dybip.componentInstance.BooleanData;
import dybip.componentInstance.IntegerData;
import dybip.expression.Expression;
import dybip.expression.list.DataList;
import dybip.expression.literal.BooleanLiteral;
import dybip.expression.literal.IntegerLiteral;
import dybip.expression.literal.NullLiteral;
import dybip.expression.variable.Index;
import dybip.Context;

public class Assignment extends BinaryActionExpression{
	public Assignment(Expression e1, Expression e2){
		super(e1, e2);
	}

	@Override
	public Expression minify(Context context){

		Expression e1 = expr1.minify(context);
		Expression e2 = expr2.minify(context);
		
		Class<?> lhsClass = e1.getExpressionType(context);
		Class<?> rhsClass = e2.getExpressionType(context);
		
		if(e1 instanceof DataList) // a component data access update statement
			return componentDataAccessAssignment(context);
		else{
			if(lhsClass == Integer.class && rhsClass == Integer.class) 
			{ // index variable or parameter variable or INtegerdata pass by value
				return integerAssignment(context);
			}	
			else if(lhsClass == Boolean.class && rhsClass == Boolean.class) // boolean data
			{ // pass by value
				return booleanAssignment(context);
			}
			else if(rhsClass == NullLiteral.class){
				// TODO assignment to null
				return new BooleanLiteral(true);
			}
				
		}		
		return new BooleanLiteral(false);
	}
	public Expression booleanAssignment(Context context)
	{
		Expression lhs = expr1.minify(context);
		if(lhs instanceof BooleanData){
			Boolean temp = expr2.minify(context).value(Boolean.class);
			BooleanLiteral bl = new BooleanLiteral(temp);
			((BooleanData) lhs).setData(bl);
			return new BooleanLiteral("true");
		}
		return new BooleanLiteral("false"); // ERROR
	}
	public Expression integerAssignment(Context context){
		Expression lhs = expr1.minify(context);
		if(lhs instanceof IntegerData){
			Integer temp = expr2.minify(context).value(Integer.class);
			IntegerLiteral il = new IntegerLiteral(temp);
			((IntegerData) lhs).setData(il);
			return new BooleanLiteral("true");
		}
//		else if(lhs instanceof Parameter){
//			Parameter para = (Parameter)lhs;
//			Integer temp = expr2.minify(context).value(Integer.class);
//			Integer il = new Integer(temp);
//			para.setValue(il);
//			return new BooleanLiteral("true");
//		}
		else if(lhs instanceof Index){
			Index index = (Index)lhs;
			Integer temp = expr2.minify(context).value(Integer.class);
			Integer il = new Integer(temp);
			index.setValue(il);
			return new BooleanLiteral("true");
		}
		return new BooleanLiteral("false"); // ERROR
	}
	public Expression componentDataAccessAssignment(Context context){
		
		Expression rhs  = expr2.minify(context);
		DataList lhs = ((DataList)expr1.minify(context));
		Class<?> lhsClass = lhs.getListType(context);
		Class<?> rhsClass = rhs.getExpressionType(context);
		
		if(rhsClass == NullLiteral.class){ // if right hand side is null literal 
			for(int i = 0; i < lhs.size(); i++){
				lhs.get(i).resetData();
			}
			return new BooleanLiteral(true);
		}
		else if(rhs instanceof DataList){
			// ASSUMPTION pass by value for boolean and integer datalist
			DataList dlist = (DataList)rhs;
			Class<?> dlistclass = dlist.getListType(context);
			// datalist Integer
			if(lhsClass == Integer.class  && dlistclass == Integer.class && dlist.size() == 1){
				// ASSUMPTION takes value of first element
				IntegerLiteral value = ((IntegerData) dlist.get(0)).getInteger();
				lhs.setData(value);
				return new BooleanLiteral(true);
			}
			// datalist Boolean
			else if(lhsClass == Boolean.class && dlistclass == Boolean.class && dlist.size() == 1){
				// ASSUMPTION takes value of first element
				BooleanLiteral value = ((BooleanData) dlist.get(0)).getBoolean();
				lhs.setData(value);
				return new BooleanLiteral(true);
			}
			return new BooleanLiteral(false); // ERROR 
		}
		else if(rhsClass == Integer.class)
		{
			Integer intTemp = ((IntegerLiteral)rhs).value(Integer.class);
			IntegerLiteral value = new IntegerLiteral(intTemp);
			lhs.setData(value);
			return new BooleanLiteral(true);
		}
		else if(rhsClass == Boolean.class){
			Boolean intTemp = ((BooleanLiteral)rhs).minify(context).value(Boolean.class);
			BooleanLiteral value= new BooleanLiteral(intTemp);
			lhs.setData(value);
			return new BooleanLiteral(true);
		}
		return new BooleanLiteral(false); // ERROR 
	}
	
	@Override
	public String toString(){
		return expr1.toString() + " := " + expr2.toString();
	}
	@Override
	public String toMinifyString(Context context){
		return expr1.toMinifyString(context) + " := " + expr2.toMinifyString(context);
	}
}
