package dybip.expression.action;

import dybip.Context;
import dybip.GroupList;
import dybip.componentInstance.ComponentInstance;
import dybip.expression.Expression;
import dybip.expression.literal.BooleanLiteral;
import dybip.motifInstance.Group;
import dybip.motifInstance.MotifInstance;
import dybip.system.SystemContext;

public class DeleteAction extends UnaryActionExpression{

	public DeleteAction(Expression expr){
		super(expr);
	}
	@Override
	public Expression minify(Context context) {
		// remove element from all groups it is in 
		Expression minifiedExpr = getExpr().minify(context);
		
		if(minifiedExpr instanceof ComponentInstance) {
			deleteComponent((ComponentInstance)minifiedExpr);
		}
		else if(minifiedExpr instanceof Group) {
			deleteGroup((Group)minifiedExpr);
		}
		else if(minifiedExpr instanceof MotifInstance) {
			deleteMotif((MotifInstance) minifiedExpr, (SystemContext)context);
		}
		System.gc();
		return new BooleanLiteral(false);
	}
	private void deleteComponent(ComponentInstance cInstance) {
		// to delete component remove from all groups it is in
		GroupList groupList = cInstance.getGroupPtrList();
		for(Group group: groupList.getArrayList()){
			group.remove(cInstance);
		}
		cInstance.resetGroupPtrList();
	}
	private void deleteGroup(Group group) {
		// to delete group
		// 1) delete all elements in group
		// 2) remove reference to this group from motif Context
		for(int i = 0; i < group.size(); i++){
			deleteComponent(group.get(i));
		}
	}
	private void deleteMotif(MotifInstance mInstance, Context context) {
		// to delete motif
		// 1) delete all groups and its elements
		// 2) remove reference to the motif from system context 
		GroupList groupList = mInstance.getGroups();
		for(int i = 0; i < groupList.size(); i++) {
			deleteGroup(groupList.get(i));
			mInstance.removeGroup(groupList.get(i));
		}
		
		String type = mInstance.getTypeStr();
		((SystemContext)context).removeMotif(type, mInstance.getName());
	}
	public String toString(){ 
		return "delete(" + getExpr().toString() +")"; 
	}
	@Override
	public String toMinifyString(Context context) {
		return "delete(" + getExpr().toMinifyString(context) +")"; 
	}
	public String toString(int toStringMode){
		return "";
	}
}
