package dybip.expression.binary;

import java.util.Random;

import dybip.expression.Expression;
import dybip.expression.list.DataList;
import dybip.expression.literal.IntegerLiteral;
import dybip.motifInstance.Group;
import dybip.Context;
import dybip.componentInstance.ComponentInstance;

public class BinaryFunction extends BinaryExpression{
	String function;
	public BinaryFunction(Expression e1, String function, Expression e2){
		super(e1,e2);
		this.function = function;
	}

	@Override
	public Expression minify(Context context){
		Expression e1 = expr1.minify(context);
		Expression e2 = expr2.minify(context);
		
		Class<?> c1 = e1.getExpressionType(context);
		Class<?> c2 = e2.getExpressionType(context);
		
		if(e1 instanceof Group & e2 instanceof Group){
			Group cinstanceGroup = (Group) e1;
			Group group = (Group)e2;
			
			if(cinstanceGroup.size() == 1)
			{
				ComponentInstance cinstance = (ComponentInstance) cinstanceGroup.get(0);
				int index = group.indexOf(cinstance);
				return new IntegerLiteral(index);
			}
			return new IntegerLiteral(-1);
		}
		
		if(c1 == Integer.class && c2 == Integer.class)
		{
			Integer i1 = e1.value(Integer.class);
			Integer i2 = e2.value(Integer.class); 
		
			return new IntegerLiteral(function(i1, i2));
		}
		else if(c1 == DataList.class && c2 == DataList.class){
			DataList l1 = (DataList) e1;
			DataList l2 = (DataList) e2;
			
			return new IntegerLiteral(functionOnDataListPair(l1, l2, context));
		}
		else if(c1 == DataList.class && c2 == Integer.class){
			DataList l1 = (DataList) e1;
			Integer i2 = e2.value(Integer.class); 
			
			Integer i1 = functionOnDataList(l1, context);
			return  new IntegerLiteral(function(i1, i2));
		}
		else if(c1 == Integer.class && c2 == DataList.class){
			DataList l1 = (DataList) e2;
			Integer i1 = e1.value(Integer.class); 
			
			Integer i2 = functionOnDataList(l1, context);
			return  new IntegerLiteral(function(i1, i2));
		}
		System.err.println("BInary function not applicable to non integer expressions " + expr1 + " " + expr2);
		// ERROR not applicable
		return null;
	}
	public Integer functionOnDataListPair(DataList l1, DataList l2, Context context){
		Integer i1 = functionOnDataList(l1, context);
		Integer i2 = functionOnDataList(l2, context);
		// WHat if one of them is null 
		return function(i1, i2);
	}
	public Integer functionOnDataList(DataList list, Context context){
		if(list.getListType(context) == Integer.class)
			return null;  //ERROR applicable on integer datalists only
		else if(list.size() == 0)
			return null; // ERROR EMPTY LIST
		else if(list.size() == 1)
			return list.get(0).minify(context).value(Integer.class);
		else{
			Integer i1 = list.get(0).minify(context).value(Integer.class);
			Integer i2 = list.get(1).minify(context).value(Integer.class);
			Integer temp = function(i1, i2);
			for(int i = 2; i < list.size(); i++){
				temp = function(temp,list.get(i).minify(context).value(Integer.class));
			}
			return temp;
		}
	}
	public Integer function(Integer i1, Integer i2){
		if(i1 == null && i2 == null)
			return null; // ERROR null integers
		else if(i1 == null && i2 != null)
			return i2;
		else if(i1 != null && i2 == null)
			return i1;
		else 
			switch (function){
				case "max": return Math.max(i1, i2);
				case "min": return Math.min(i1, i2);
				case "avg": return (i1 + i2)/2;
				case "mod": return i1 % i2;
				case "random": return i1 + new Random().nextInt(i2 -i1 + 1);
				default: return null; // ERROR not applicable function 
				
			}
	}
	@Override
	public String toString(){
		return function + "(" + expr1.toString() + ", " + expr2.toString() + ")";
	}
	@Override
	public String toMinifyString(Context context){
		return function + "(" + expr1.toMinifyString(context) + ", " + expr2.toMinifyString(context) + ")";
	}
}
