package dybip.expression.binary;


import dybip.componentInstance.IntegerData;
import dybip.expression.Expression;
import dybip.expression.list.DataList;
import dybip.expression.literal.BooleanLiteral;
import dybip.expression.literal.IntegerLiteral;
import dybip.Context;

public class MathmeticalOperation extends BinaryExpression{

	private String operator;
	public MathmeticalOperation(Expression e1, String operator, Expression e2){
		super(e1, e2);
		this.operator = operator;
	}

	@Override
	public String toString(){
		return expr1.toString() + " " + operator + " " + expr2.toString();
	}
	@Override
	public String toMinifyString(Context context){
		return expr1.toMinifyString(context) + " " + operator + " " + expr2.toMinifyString(context);
	}

	public Expression minify(Context context){
		Expression e1 = (expr1.minify(context));
		Expression e2 = (expr2.minify(context));

	
		Class<?> c1 = e1.getExpressionType(context);
		Class<?> c2 = e2.getExpressionType(context);

		// ERROR operation on 
		if(c1==Double.class && c2 == Integer.class) {
			double i1 = e1.value(Double.class);
			int i2 = e2.value(Integer.class); 

			switch (operator){
			case "-": return new IntegerLiteral(i1 - i2);
			case "+": return new IntegerLiteral(i1 + i2);
			case "*": return new IntegerLiteral(i1 * i2);
			case "/": return new IntegerLiteral(i1 / i2);
			case "%": return new IntegerLiteral(i1 % i2);}
		}
		// ERROR operation on 
		else if(c1 == Integer.class && c2 == Integer.class)
		{
			int i1 = e1.value(Integer.class);
			int i2 = e2.value(Integer.class); 

			switch (operator){
			case "-": return new IntegerLiteral(i1 - i2);
			case "+": return new IntegerLiteral(i1 + i2);
			case "*": return new IntegerLiteral(i1 * i2);
			case "/": return new IntegerLiteral(i1 / i2);
			case "%": return new IntegerLiteral(i1 % i2);
			case "^": return new IntegerLiteral((int)Math.pow(i1, i2));
			}
		}
		else if(c1 == Boolean.class && c2 == Boolean.class){
			Boolean orResult = e1.value(Boolean.class) | e2.value(Boolean.class);
			return new BooleanLiteral(orResult);
		}
		else if(c1 == DataList.class && c2 == DataList.class)
		{
			DataList list1 = (DataList)e1;
			DataList list2 = (DataList)e2;
			if(list1.getListType(context) == Integer.class && list2.getListType(context) == Integer.class)
			{
				Integer temp = 0;
				for(int i = 0; i < list1.size(); i++){
					temp += ((IntegerData) list1.get(i)).minify(context).value(Integer.class);
				}
				for(int i = 0; i < list2.size(); i++){
					temp += ((IntegerData) list2.get(i)).minify(context).value(Integer.class);
				}
				return new IntegerLiteral(temp);
			}

		}
		//		else if( c1 == ComponentArrayInstance.class && c2 == ComponentArrayInstance.class){
		//			ComponentArrayInstance list1 = (ComponentArrayInstance)e1;
		//			ComponentArrayInstance list2 = (ComponentArrayInstance)e2;
		//			switch (operator){
		//				case "-": 
		//					ComponentArrayInstance cai = new ComponentArrayInstance(list1.getstr(), list1.size()-list2.size());
		//					for(int i = 0; i < list1.size(); i++){
		//						if(list2.contains(list1.get(i)))
		//								;
		//						else							
		//							cai.add(list1.get(i));
		//					}
		//					return cai;
		//				case "+": 
		//					// TODO duplicates what if 
		//					 cai = new ComponentArrayInstance(list1.getstr(), list1.size()+list2.size());
		//					
		//					 for(int i = 0; i < list1.size(); i++)				
		//							cai.add(list1.get(i));
		//					
		//					for(int i = 0; i < list2.size(); i++)		
		//						if(!list1.contains(list2.get(i)))
		//							cai.add(list2.get(i));
		//					
		//					return cai;
		//				
		//				default: // ERROR uncompatible type
		//			}
		//		}
		return null;
	}
	@Override
	public Class<?> getExpressionType(Context context) {
		return Integer.class;
	}


}
