package dybip.expression.booleanExpr;

import dybip.expression.Expression;
import dybip.expression.list.DataList;
import dybip.expression.literal.BooleanLiteral;
import dybip.expression.literal.NullLiteral;
import dybip.motifInstance.Group;
import dybip.motifInstance.MotifInstance;
import dybip.Context;
import dybip.componentInstance.ComponentInstance;

public class Equality extends BinaryBooleanExpression{
	public Equality(Expression e1, Expression e2){
		super(e1, e2);
	}
	@Override
	public String toString(){
		return expr1.toString() + " = " + expr2.toString();
	}
	@Override
	public String toMinifyString(Context context){
		return expr1.toMinifyString(context) + " = " + expr2.toMinifyString(context);
	}
	@Override
	public Expression minify(Context context){
		// left hand side
		Expression lhs = expr1.minify(context);
		Class<?> c1 = lhs.getExpressionType(context);
		// right hand side
		Expression rhs = expr2.minify(context);
		Class<?> c2 = rhs.getExpressionType(context);
		
		if(c1 == Integer.class && c2 == Integer.class)
		{
			int e1 = lhs.value(Integer.class);
			int e2 = rhs.value(Integer.class); 
			return new BooleanLiteral(e1 == e2);
		}
		else if(lhs instanceof DataList && (lhs.asDataList().size() > 1)){
			if(rhs instanceof NullLiteral){
				DataList dlist = (DataList) lhs;
				boolean equal = true;
				for(int i = 0; i < dlist.size(); i++)
					equal = equal & dlist.get(i) == null;
				return new BooleanLiteral(equal);
			}
			else if(rhs instanceof ComponentInstance){
//				ComponentInstance ci = (ComponentInstance) rhs;
//				
//				DataList dlist = (DataList) lhs;
//				boolean equal = true;
//				for(int i = 0; i < dlist.size(); i++)
//					// ASSUMPTION check if all datalist elements have same reference as first component in array
//					equal = equal & ((ComponentInstance) dlist.get(i)).getCi() == ci.get(0);
//				return new BooleanLiteral(equal);
			}
			// compare list to single elem, null, boolean, integer
			else if(rhs instanceof BooleanLiteral) {
				DataList dlist = (DataList) lhs;
				
				boolean equal = true;
				for(int i = 0; i < dlist.size(); i++) {
					boolean left = dlist.get(i).value(Boolean.class);
					left = left == rhs.value(Boolean.class);
					equal = equal & left; 
				}
				return new BooleanLiteral(equal);
			}
		}
		else if(c1 == Boolean.class && c2 == Boolean.class){
			Boolean b1 = lhs.value(Boolean.class);
			Boolean b2 = rhs.value(Boolean.class);
			return new BooleanLiteral(b1.equals(b2));
		}
		else if(c1 == Group.class && c2 == Group.class)
		{
			Group caiLeft = (Group) lhs;
			Group caiRight = (Group) rhs;
			
			if(caiLeft.size() != caiRight.size())
				return new BooleanLiteral(false);
			else{
				for(int i = 0; i < caiLeft.size(); i++) // DO I CARE ABOUT ORDER?
					if(caiLeft.get(i) != caiRight.get(i))
						return new BooleanLiteral(false);
				return new BooleanLiteral(true);
			}
		}
		else if(c1 == MotifInstance.class && c2 == MotifInstance.class)
		{
			return new BooleanLiteral(lhs == rhs);
		}
		
		
//		// compare two lists
//		else if (c1 == DataList.class && 
//				c2 == DataList.class &&
//				((DataList)lhs).getListType(context) == ComponentArrayInstance.class &&
//				((DataList)rhs).getListType(context) == ComponentArrayInstance.class)
//			return new BooleanLiteral(lhs == rhs);
		// ERROR type mis match or invalid type
		return null;
	}

}
