package dybip.expression.list;

import java.util.ArrayList;
import java.util.ListIterator;

import dybip.expression.Expression;
import dybip.expression.literal.BooleanLiteral;
import dybip.expression.variable.PortAccess;
import dybip.Context;

public class ExpressionList<T extends Expression> extends Expression{
	private  ArrayList<T> arrayList;
	public ExpressionList(int size){
		arrayList = new ArrayList<T>(size);
	}
	public ExpressionList(){
		arrayList = new ArrayList<T>();
	}

	public int size(){
		return arrayList.size();
	}
	public ListIterator<T> listIterator(){
		return arrayList.listIterator();
	}

	public T get(int index){
		return arrayList.get(index); 
	}
	public void set(int index, T item){
		arrayList.set(index, item);
	}
	public void setList(ArrayList<T> arrayList){
		this.arrayList =  arrayList;
	}
	public boolean contains(T elem){
		return arrayList.contains(elem);
	}
	
	
	public ArrayList<T> getArrayList(){
		return arrayList;
	}
	
	public void add(T elem){
		arrayList.add(elem);
	}
	public void addAll(ArrayList<T> list){
		arrayList.addAll(arrayList);
	}
	
	
	public int indexOf(T object) {
		return arrayList.indexOf(object);
	}
	public void remove(int index){
		arrayList.remove(index);
	}
	public void removeAll(ExpressionList<T> list) {
		arrayList.removeAll(list.getArrayList());
	}
	public void remove(int beginIndex, int endIndex) {
		for(int i = beginIndex; i <= endIndex; i++) {
			remove(i);
		}
	}
	
	
	@Override 
	public String toString(){
		String str = "";
		for(int i = 0; i < size(); i++)
			str += get(i) + "\t";
		return str;
	}
	@Override 
	public String toMinifyString(Context context){
		String str = "";
		for(int i = 0; i < size(); i++)
			str += get(i).toMinifyString(context) + "\t";
		return str;
	}
	@Override
	public Expression minify(Context context) {
		Boolean completed = true;
		for(int i = 0; i < size(); i++){
			completed = completed && get(i).minify(context).value(Boolean.class);
		}
		return new BooleanLiteral(completed);
	}
	@Override
	public Class<?> getExpressionType(Context context) {
		return get(0).getExpressionType(context);
	}
}
