package dybip.expression.unary;

import dybip.expression.Expression;
import dybip.expression.list.DataList;
import dybip.expression.literal.*;
import dybip.Context;

public class UnaryFunction extends UnaryExpression{
	
	String operator;
	
	public UnaryFunction(String operator, Expression expr){
		super(expr);
		this.operator = operator;
	}
	@Override
	public Expression minify(Context context){
		Expression minified = expr.minify(context);
		if(minified instanceof DataList){
			DataList list = (DataList) minified;
			return new IntegerLiteral(functionOnDataList(list, context));
		}
		else if(minified.getExpressionType(context)== Integer.class)
			return function(minified.value(Integer.class));
		else
			return null; // ERROR function not applicable on type
	}
	public Expression function(Integer i) {
		if(operator.equals("max")|operator.equals("min")|operator.equals("avg")|operator.equals("mod"))
			return new IntegerLiteral(i); 
		else if(operator.equals("sin")|operator.equals("cos"))
			switch (operator){
				case "sin": return new DecimalLiteral(Math.sin(Math.toRadians(i)));
				case "cos": return new DecimalLiteral(Math.cos(Math.toRadians(i)));
			}
		return null;
	}
	public Integer functionOnDataList(DataList list, Context context){
		if(list.getListType(context) != Integer.class)
			return null;  //ERROR applicable on integer datalists only
		else if(list.size() == 0)
			return null; // ERROR EMPTY LIST
		else if(list.size() == 1)
			return list.get(0).minify(context).value(Integer.class);
		else{
			Integer i1 = list.get(0).minify(context).value(Integer.class);
			Integer i2 = list.get(1).minify(context).value(Integer.class);
			Integer temp = function(i1, i2);
			for(int i = 2; i < list.size(); i++){
				temp = function(temp,list.get(i).minify(context).value(Integer.class));
			}
			return temp;
		}
	}
	public Integer function(Integer i1, Integer i2){
		if(i1 == null && i2 == null)
			return null; // ERROR null integers
		else if(i1 == null && i2 != null)
			return i2;
		else if(i1 != null && i2 == null)
			return i1;
		else 
			switch (operator){
				case "max": return Math.max(i1, i2);
				case "min": return Math.min(i1, i2);
				case "avg": return (i1 + i2)/2;
				case "mod": return i1 % i2;
				default: return null; // ERROR not applicable function 
			}
	}
	@Override
	public String toString(){
		return operator + "(" + expr.toString() + ")";
	}
	@Override
	public String toMinifyString(Context context){
		return operator + "(" + expr.toMinifyString(context) + ")";
	}
}
