package dybip.expression.variable;

import dybip.Context;
import dybip.componentInstance.ComponentInstance;
import dybip.expression.Expression;
import dybip.expression.list.DataList;
import dybip.expression.list.ExpressionList;
import dybip.motifInstance.Group;
import dybip.motifInstance.MotifInstance;

public class VariableList extends ExpressionList<VariableExpression>{
	public VariableList(int size){
		super(size);
	}
	public VariableList(){
		super();
	}
	@Override
	public Expression minify(Context context){

		Expression firstExpr = get(0).minify(context);
		Class<?> classType = firstExpr.getExpressionType(context);
		
		if(classType == Group.class) {
			Group group = (Group) firstExpr;
			if(size() == 1) // return group either group.size expr or group.port
       			return group;
			else // data access in group
			{
				DataList dList = new DataList(group.size());
				// get data variable name 
				VariableExpression varExpression = (VariableExpression) get(1);
				String dataVarName = varExpression.getVariableName();
				for(int i = 0; i < group.size(); i++) {
					ComponentInstance cInstance = group.get(i);
					dList.add(cInstance.getData(dataVarName));
				}
				return dList;
			}
		}
		else if(classType == MotifInstance.class) {
			// TODO
			MotifInstance minstance = (MotifInstance) firstExpr;
			if(size() == 2) {
				// return group in motif
				// return element in motif group
				
				Expression secondVarExpr = get(1);
				String groupName = ((VariableExpression) get(1)).getVariableName();
				Group group =  minstance.getGroup(groupName);
				
				if(secondVarExpr instanceof GroupAccess) {
					GroupAccess groupAccess = (GroupAccess) secondVarExpr;
					Expression indexExpr = groupAccess.getIndexExpr().minify(context);
					Integer index = indexExpr.value(Integer.class);
					
					if(index >  group.size() || index < 0 ){// ERROR index out of bound
						System.err.println("INDEX OUT OF BOUND"); System.exit(0);}
					else{
						ComponentInstance ci = group.get(index);
						Group newGroup = new Group(1, group.getName()); // TODO: what to name group?
						newGroup.add(ci);
						return newGroup;
					}
				}
				else if(secondVarExpr instanceof SubGroupAccess) {
					SubGroupAccess subGroupAccess = (SubGroupAccess) secondVarExpr;
					
					
					Expression beginIndex = subGroupAccess.getBeginIndexExpr().minify(context);
					Expression lastIndex = subGroupAccess.getEndIndexExpr().minify(context);
					Integer startIndex = beginIndex.value(Integer.class);
					Integer endIndex = lastIndex.value(Integer.class);
					
					if(endIndex >  group.size() || startIndex < 0 ){
						System.err.println("INDEX OUT OF BOUND"); System.exit(0);}
					else{
						
						Group newGroup = new Group(endIndex - startIndex + 1, group.getName()); // TODO: what to name group?
						for(int i = startIndex; i <= endIndex; i++){
							ComponentInstance ci = group.get(i);
							newGroup.add(ci);
						}
						return newGroup;
					}
				}
				
				return group;
				
				// What if element access
			}
			else if(size() == 3){
				// accessing data in motif group/ group element
				String groupName = ((VariableExpression) get(1)).getVariableName();
				Group group =  minstance.getGroup(groupName);
				Expression secondVarExpr = get(1);
				
				
				if(secondVarExpr instanceof GroupAccess) {
					GroupAccess groupAccess = (GroupAccess) secondVarExpr;
					Expression indexExpr = groupAccess.getIndexExpr().minify(context);
					Integer index = indexExpr.value(Integer.class);//context.getIndex(indexExpr);
					
					if(index >  group.size() || index < 0 ){// ERROR index out of bound
						System.err.println("INDEX OUT OF BOUND"); System.exit(0);}
					else{
						ComponentInstance ci = group.get(index);
						group = new Group(1, group.getName()); // TODO: what to name group?
						group.add(ci);
					}
				}
				else if(secondVarExpr instanceof SubGroupAccess) {
					SubGroupAccess subGroupAccess = (SubGroupAccess) secondVarExpr;
					
					Expression beginIndex = subGroupAccess.getBeginIndexExpr().minify(context);
					Expression lastIndex = subGroupAccess.getEndIndexExpr().minify(context);
					Integer startIndex = beginIndex.value(Integer.class);
					Integer endIndex = lastIndex.value(Integer.class);
					
					if(endIndex >  group.size() || startIndex < 0 ){
						System.err.println("INDEX OUT OF BOUND"); System.exit(0);}
					else{
						
						group = new Group(endIndex - startIndex + 1, group.getName()); // TODO: what to name group?
						for(int i = startIndex; i <= endIndex; i++){
							ComponentInstance ci = group.get(i);
							group.add(ci);
						}
					}
					
				}
				
				DataList dList = new DataList(group.size());
				// get data variable name 
				VariableExpression varExpression = (VariableExpression) get(2);
				String dataVarName = varExpression.getVariableName();
				for(int i = 0; i < group.size(); i++) {
					ComponentInstance cInstance = group.get(i);
					dList.add(cInstance.getData(dataVarName));
				}
				return dList;
				
			}
		}
		else if(classType == ComponentInstance.class) {
			//accessing data or port
		}
		return null;
		
	}
	@Override
	public String toString(){
		if(size() == 0)
			return "";
		String result = "";
		for(int i = 0 ; i < size(); i++)
			if(i == 0)
				result += get(i).toString();
			else
				result += "." + get(i).toString();
		
		return result;
	}
	@Override
	public String toString( int toStringMode){
		if(size() == 0)
			return "";
		String result = "";
		for(int i = 0 ; i < size(); i++)
			if(i == 0)
				result += get(i).toString(toStringMode);
			else
				result += "." + get(i).toString(toStringMode);
		
		return result;
	}
	@Override
	public Class<?> getExpressionType(Context context) {
		// TODO Auto-generated method stub
		return null;
	}
	@Override
	public String toMinifyString(Context context) {
		// TODO Auto-generated method stub
		return null;
	}

}
