package dybip.motifInstance;

import java.util.HashSet;
import java.util.Set;

import dybip.Context;
import dybip.GroupList;
import dybip.componentInstance.ComponentInstance;
import dybip.componentInstance.DataMap;
import dybip.componentType.ComponentType;
import dybip.componentType.ComponentTypeMap;
import dybip.expression.Expression;
import dybip.expression.variable.GroupAccess;
import dybip.expression.variable.SubGroupAccess;
import dybip.expression.variable.VariableExpression;
import dybip.motifType.MotifType;
import dybip.systemStmt.motifTypeStmt.ComponentInstanceStmt;
import dybip.systemStmt.motifTypeStmt.ComponentInstanceStmtList;

public class MotifContext extends Context{
	private ComponentGroupMap groupMap;
	private ComponentTypeMap ctmap;
	private DataMap sysContextDMap;

	public MotifContext(MotifType motifType, ComponentTypeMap ctmap, DataMap sysContextMap){
		this.ctmap = ctmap;
		bindComponentInstances(motifType.getComponentInstanceStmtList(), ctmap);
		this.sysContextDMap = sysContextMap;
	}

	public void bindComponentInstances(ComponentInstanceStmtList cList, ComponentTypeMap ctmap){
		groupMap = new ComponentGroupMap(cList.size()); // initialize count of component groups in motif 
		for(int i = 0; i < cList.size(); i++){
			ComponentInstanceStmt instanceStmt = cList.get(i);
			bindComponentInstance(instanceStmt, ctmap);
		}
	}
	public void bindComponentInstance(ComponentInstanceStmt instanceStmt, ComponentTypeMap ctmap){
		ComponentType ct = ctmap.get(instanceStmt.getType());
		Expression exprVar = instanceStmt.getVariableExpression();
		String groupName = ((VariableExpression)exprVar).getVariableName();

		if(exprVar instanceof GroupAccess){
			Expression indexExpr = ((GroupAccess)exprVar).getIndexExpr();
			Expression minifiedExpr = indexExpr.minify(this);
			int groupSize  = (Integer) minifiedExpr.value(Integer.class);
			// TODO evaluate expression count what if not int 

			createInitilizeGroup(groupName, groupSize, ct);
		}
		else if(exprVar instanceof SubGroupAccess){
			// TODO  report error cant create such an instance 
		}
		else if(exprVar instanceof VariableExpression){
			createInitilizeGroup(groupName, 1, ct);
		}
	}
	public void createInitilizeGroup(String groupName, int groupSize, ComponentType ct){

		Group group = new Group(groupSize, groupName);
		for(int i = 0; i < groupSize; i++){
			ComponentInstance temp = new ComponentInstance(group, ct, this); // no current state
			group.add(temp);
			//TODO: initial action and gaurd must be evaluated before moving to current space
		}
		groupMap.put(groupName, group);
	}

	public void addGroup(Group group){
		groupMap.put(group.getName(), group);
	}
	public Group getGroup(String name) {
		if(groupMap.containsKey(name)) {
			return groupMap.get(name);
		}
		else{
			System.err.println("No such Group Exists " + name);
			return null;
		}
	}
	public void removeGroup(Group group) {
		groupMap.remove(group.getName());
	}
	public GroupList getGroups() {
		GroupList groupList = new GroupList();
		for(String groupName: groupMap.keySet()) {
			groupList.add(groupMap.get(groupName));
		}
		return groupList;
	}

	public String trackMotifSize(int step, String instName, int runCount) {
		String size = "";
		//		for(String groupName: groupMap.keySet())
		
		if(groupMap.containsKey("cars")) {
			Group g = groupMap.get("cars");
			if(g.size() > 0)
				size += runCount + "\t" + instName + "\t" + step + "\t" + g.size() + "\t" + g.get(0).getData("pos") + "\t";
			else
				size += runCount + "\t" + instName + "\t" + step + "\t" + g.size() + "\t";
			for(int i = 0; i < g.size(); i++)
			{
				ComponentInstance ci = g.get(i);
				size += ci.getID() + "," + ci.getData("pos")+ "\t";
			}
			return size + "\n";
		}
		else if(groupMap.containsKey("nest")) {
			Group g = groupMap.get("nest");
			for(int i = 0; i < g.size(); i++){
				ComponentInstance ci = g.get(i);
				size += ci.getData("x") + "," + ci.getData("y")+ "\t";
			}	
			if(groupMap.containsKey("prey")) {
				g = groupMap.get("prey");
				for(int i = 0; i < g.size(); i++){
					ComponentInstance ci = g.get(i);
					size += ci.getData("x") + "," + ci.getData("y")+ "\t";
				}
			}
			if(groupMap.containsKey("activebots")) {
				g = groupMap.get("activebots");
				for(int i = 0; i < g.size(); i++){
					ComponentInstance ci = g.get(i);
					size += ci.getData("x") + "," + ci.getData("y")+ "\t";
				}	
			}
			if(groupMap.containsKey("inactivebots")) {
				g = groupMap.get("inactivebots");
				for(int i = 0; i < g.size(); i++){
					ComponentInstance ci = g.get(i);
					size += ci.getData("x") + "," + ci.getData("y")+ "\t";
				}	
			}
			return size + "\n";
		}
		return size;
	}
	public Set<Integer> getComponentInstanceCount() {
		Set<Integer> ids = new HashSet<Integer>();
		for(String groupName: groupMap.keySet()) {
			Group group = groupMap.get(groupName);
			ids.addAll(group.getIds());
		}
		return ids;
	}
	public String getTrace() {
		return groupMap.getTrace();
	}
	@Override
	public Expression find(String varName) {
		if(groupMap.containsKey(varName))
			return groupMap.get(varName);
		else if(getIndexMap().containsKey(varName))
			return getIndexMap().get(varName);
		else if(getDataMap().containsKey(varName))
			return getDataMap().get(varName);
		else if(ctmap.containsKey(varName))
			return ctmap.get(varName);
		else if(sysContextDMap.containsKey(varName))
			return sysContextDMap.get(varName);
		else
			return null;
	}
	@Override
	public String toString(){
		return groupMap.toString() + "\t\t\tIndexMap " + getIndexMap().toString();
	}
}
