package dybip.motifInstance;

import java.util.Set;

import dybip.Context;
import dybip.GroupList;
import dybip.InteractionStmt.EnabledInteraction;
import dybip.InteractionStmt.EnabledReconfiguration;
import dybip.InteractionStmt.EnabledStep;
import dybip.InteractionStmt.EnabledStepList;
import dybip.InteractionStmt.ExecutableInteractionRuleList;
import dybip.InteractionStmt.ExecutableReconfigurationRuleList;
import dybip.InteractionStmt.InteractionRuleStmtList;
import dybip.InteractionStmt.ReconfigurationRuleStmtList;
import dybip.componentInstance.DataMap;
import dybip.componentType.ComponentTypeMap;
import dybip.expression.Expression;
import dybip.motifType.MotifType;

public class MotifInstance extends Expression{
	private String varName;
	private MotifType typePointer;
	private MotifContext context;
	
	private ExecutableInteractionRuleList executableInteractionRuleList; // interprted version of move list
	private ExecutableReconfigurationRuleList executableReconfigRuleList;
	
	private EnabledStepList<EnabledInteraction> enabledInteractions; // enabled component interactions
	private EnabledStepList<EnabledReconfiguration> enabledReconfig;
	
	private static long counter = 0;
	private final long id;
	/*
	 * CONSTRUCTOR
	 */
	public MotifInstance(MotifType motifType, ComponentTypeMap ctmap, DataMap sysContextDMap){
		this.typePointer = motifType;
		this.context = new MotifContext(motifType, ctmap, sysContextDMap); // initialize context
		createExecutableInteractionList(motifType);
		createExectableReconfigurationList(motifType);
		enabledInteractions = new EnabledStepList<EnabledInteraction>();
		enabledReconfig = new EnabledStepList<EnabledReconfiguration>();
		id = counter++;
		this.varName = motifType.getName().charAt(0) + "" + id;
	}
//	public MotifInstance(String varName, MotifType motifType){
//		this.varName = varName;
//		this.typePointer = motifType;
//		this.context = new MotifContext(motifType); // initialize context
//		createExecutableInteractionList(motifType);
//		createExectableReconfigurationList(motifType);
//		enabledInteractions = new EnabledStepList<EnabledInteraction>();
//		enabledReconfig = new EnabledStepList<EnabledReconfiguration>();
//	}
	private void createExecutableInteractionList(MotifType motifType){
		InteractionRuleStmtList interactionStmtList = motifType.getInteractionRuleStmtList();
		executableInteractionRuleList = new ExecutableInteractionRuleList(interactionStmtList, context);
	}
	private void createExectableReconfigurationList(MotifType motifyType) {
		ReconfigurationRuleStmtList reconfigSmtList = motifyType.getReconfigurationRuleStmtList();
		executableReconfigRuleList = new ExecutableReconfigurationRuleList(reconfigSmtList, context);
	}
	
	
	public EnabledStep getStep(String stepType, int index) {
		if(stepType.equals("r") || stepType.equals("R"))
			return enabledReconfig.get(index);
		else if(stepType.equals("i") || stepType.equals("I"))
			return enabledInteractions.get(index);
		else 
			System.err.println("WRONG STEP TYPE" + stepType);
		return null;
	}
	
	public String trackMotifSize(int step, String instName, int runCount) {
		return context.trackMotifSize(step, instName, runCount);
	}
	/*
	 * Evaluate enabled steps
	 */
	public void evaluteEnabledInteractions(){
		enabledInteractions = new EnabledStepList<EnabledInteraction>();
		for(int i = 0; i < executableInteractionRuleList.size(); i++){
			enabledInteractions.add(executableInteractionRuleList.get(i).evaluateEnabledInteraction(context));
		}
	}
	public void evaluateEnabledReconfigurations() {
		enabledReconfig = new EnabledStepList<EnabledReconfiguration>();
		for(int i = 0; i < executableReconfigRuleList.size(); i++)
			enabledReconfig.add(executableReconfigRuleList.get(i).evaluateEnabledReconfiguration(context));
	}
	public int getEnabledInteractionCount() {
		return enabledInteractions.size();
	}
	public int getEnabledReconfigurationCount() {
		return enabledReconfig.size();
	}
	/*
	 * Execute step
	 */
	public void execute(String stepType, int index) {
		if(stepType.equals("r") || stepType.equals("R"))
			enabledReconfig.get(index).execute(context);
		else if(stepType.equals("i") || stepType.equals("I"))
			enabledInteractions.get(index).execute(context);
		else
			System.err.println("wrong step type");
	}
	
	public boolean hasEnabledInteraction() {
		return enabledInteractions.size() != 0;
	}
	public boolean hasEnabledReconfiguration() {
		return enabledReconfig.size() != 0;
	}
	
	public String chosenStepToString(String stepType, int stepIndex) {
		String log = "CHOSEN STEP\n";
		log += "MOTIF: " +  varName + ", TYPE: ";
		
		if(stepType.equals("r") || stepType.equals("R"))
			log += "reconfiguration";
		else if(stepType.equals("i") || stepType.equals("I"))
			log += "interaction";
		log += ", INDEX: " + stepIndex;
		
		log += "\n" + getStep(stepType, stepIndex).toMinifyString(context);

		return log;
	}
	
	public Group getGroup(String name) {
		return context.getGroup(name);
	}
	public GroupList getGroups() {
		return context.getGroups();
	}
	public void removeGroup(Group group) {
		context.removeGroup(group);
	}
	
	public String getTypeStr() {
		return typePointer.getName();
	}
	public void setContext(MotifContext context){
		this.context = context;
	}
	public String getName() {
		return varName;
	}


	public Set<Integer> getComponentInstanceCount() {
		return context.getComponentInstanceCount();
	}
	public EnabledStepList<EnabledInteraction> getEnabledInteractions(){
		return enabledInteractions;
	}
	public EnabledStepList<EnabledReconfiguration> getEnabledReconfigurations(){
		return enabledReconfig;
	}
	
	
	@Override
	public Expression minify(Context context) {
		return this;
	}
	@Override
	public Class<?> getExpressionType(Context context) {
		return MotifInstance.class;
	}
	
	public String getTrace() {
		return context.getTrace().replace("REPLACE1", typePointer.getName() + id);
	}
	public long getId() {
		return id;
	}
	/*
	 * To String methods
	 */
	public String enabledInteractionsToString() {
		return getEnabledInteractions().toMinifyString(context);
	}
	public String enabledReconfigurationsToString() {
		return getEnabledReconfigurations().toMinifyString(context);
	}
	@Override
	public String toMinifyString(Context context) {
		// Treply on index Map and type to replace variable R1 with Ring(1) exp
		return null;
	}
	public String toString(){
		return "" + varName + "\t" + typePointer.getName() + "\t" + id + "\n" + context.toString() ; 
//				"\n" + executableInteractionRuleList.toString() + "\n" + executableReconfigRuleList.toString();
	}
}
