package dybip.motifInstance;

import java.util.HashMap;
import java.util.Set;

import dybip.DyBIPHashMap;
import dybip.componentInstance.ComponentInstance;
import dybip.componentInstance.DataMap;
import dybip.componentType.ComponentTypeMap;
import dybip.motifType.MotifTypeMap;
import dybip.systemStmt.motifInstanceStmt.MotifInstanceStmt;
import dybip.systemStmt.motifInstanceStmt.MotifInstanceStmtList;

public class MotifInstanceMap extends DyBIPHashMap<String, MotifInstance>{

	public MotifInstanceMap(MotifInstanceStmtList list, MotifTypeMap mtmap, ComponentTypeMap ctmap, DataMap sysContextDMap){
		super(list.size());
		for(int i = 0; i < list.size(); i++)
		{
			MotifInstanceStmt tempStmt = list.get(i);
			String type = tempStmt.getType();
			String varName = tempStmt.getVariableName();
			
			if(this.keySet().contains(varName)){
				System.err.println("Can't have two motif instances with same reference in one system");
				System.exit(0);
			}
			else{
				MotifInstance mi = new MotifInstance(mtmap.get(type), ctmap,sysContextDMap);
				this.put(mi.getName(), mi);
			}
		}
	}
	public MotifInstanceMap(MotifInstanceStmtList list, HashMap<String, String> pmap, MotifTypeMap mtmap, ComponentTypeMap ctmap, DataMap sysContextDMap){
		super();
		for(String type : pmap.keySet()) {
			int instanceCount = Integer.parseInt(pmap.get(type));
			for(int j = 0; j < instanceCount; j++) {
				MotifInstance mi = new MotifInstance( mtmap.get(type), ctmap, sysContextDMap);
				this.put(mi.getName(), mi);
			}
		}
	}
	public MotifInstanceMap(int size) {
		super(size);
	}
	public MotifInstanceMap() {
		super();
	}
	public MotifInstance getByID(long ID) {
		for(String key: keySet()) {
			MotifInstance mi = get(key);
			ComponentInstance ci = mi.getGroup("core").get(0);
			if(ci.getID() == ID)
				return mi;
		}
		return null;
	}
	
	public void evaluateEnabledStep(){
		// evaluate enabled interactions and reconfigurations
		
		for(String key: keySet()){
			get(key).evaluteEnabledInteractions();
			get(key).evaluateEnabledReconfigurations();
		}
	}
	public boolean hasEnabledInteractions(){
		for(String s : keySet()){
			if (get(s).hasEnabledInteraction() == true) return true;
		}
		
		return false;
	}
	public boolean hasEnabledReconfigurations() {
		for(String s : keySet()){
			if (get(s).hasEnabledReconfiguration() == true) return true;
		}
		
		return false;
	}

	public String getTrace() {
		String trace = "";
		for(String motifName: keySet()) {
			trace += get(motifName).getTrace() + "\n";
		}
		return trace;
	}
	public String toString(){
		String result = "";
		Set<String> cTypesName = keySet();
		for(String s : cTypesName)
			result += "\t\t" + s + "=" + get(s).toString() + "\n";
		return result;
	}
}
