package dybip.motifInstance;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;

import dybip.DyBIPHashMap;
import dybip.MotifInstanceCountMap;
import dybip.ParameterMap;
import dybip.componentInstance.DataMap;
import dybip.componentType.ComponentTypeMap;
import dybip.motifType.MotifTypeMap;
import dybip.system.SystemContext;
import dybip.systemStmt.motifInstanceStmt.MotifInstanceStmt;
import dybip.systemStmt.motifInstanceStmt.MotifInstanceStmtList;

public class MotifTypeInstanceMap extends DyBIPHashMap<String, MotifInstanceMap>{

	public MotifTypeInstanceMap(MotifInstanceStmtList list, MotifTypeMap mtmap, ComponentTypeMap ctmap, DataMap sysContextDMap){
		super();
		for(int i = 0; i < list.size(); i++)
		{
			MotifInstanceStmt tempStmt = list.get(i);
			String type = tempStmt.getType();
			String varName = tempStmt.getVariableName();
			
			if(this.keySet().contains(type))
			{
				MotifInstanceMap mimap = get(type);
				if(mimap.keySet().contains(varName)){
					System.err.println("Can't have two motif instances with same reference in one system");
					System.exit(0);
				}
				else {
					MotifInstance mi = new MotifInstance( mtmap.get(type), ctmap, sysContextDMap);
					mimap.put(mi.getName(), mi);
				}
			}
			else
			{				
				MotifInstanceMap mimap = new MotifInstanceMap();
				
				if(mimap.keySet().contains(varName)){
					System.err.println("Can't have two motif instances with same reference in one system");
					System.exit(0);
				}
				else{
					MotifInstance mi = new MotifInstance( mtmap.get(type), ctmap, sysContextDMap);
					mimap.put(mi.getName(), mi);
				}
				put(type, mimap);
			}
			
		}
	}
	public MotifTypeInstanceMap(MotifInstanceStmtList list, MotifInstanceCountMap micmap, MotifTypeMap mtmap, ComponentTypeMap ctmap, SystemContext context){
		super();
		for(String type : micmap.keySet()) {
			int instanceCount = (micmap.get(type)).minify(context).value(Integer.class);
			MotifInstanceMap mimap = new MotifInstanceMap(instanceCount);
			
			for(int j = 0; j < instanceCount; j++) {
				MotifInstance mi = new MotifInstance( mtmap.get(type), ctmap, context.getDataMap());
				mimap.put(mi.getName(), mi);
			}
			put(type, mimap);
		}
	}
	
	public int getMotifInstanceCount() {
		int count = 0;
		for(String type: keySet()) {
			count += get(type).size();
		}
		return count;
	}
	public int getComponentInstanceCount() {
		int count = 0;
		Set<Integer> ids = new HashSet<Integer>();
		for(String type: keySet()) {
			MotifInstanceMap mInstanceMap =  get(type);
			for(String motifName:mInstanceMap.keySet()) {
				MotifInstance minstance = mInstanceMap.get(motifName);
				ids.addAll(minstance.getComponentInstanceCount());
			}
		}
		return ids.size();
	}
	
	public int getEnabledInteractionCount() {
		int count = 0;
		for(String type: keySet()) {
			MotifInstanceMap mInstanceMap =  get(type);
			for(String motifName:mInstanceMap.keySet()) {
				MotifInstance minstance = mInstanceMap.get(motifName);
				count += minstance.getEnabledInteractionCount();
			}
		}
		return count;
	}
	public int getEnabledReconfigurationCount() {
		int count = 0;
		for(String type: keySet()) {
			MotifInstanceMap mInstanceMap =  get(type);
			for(String motifName:mInstanceMap.keySet()) {
				MotifInstance minstance = mInstanceMap.get(motifName);
				count += minstance.getEnabledReconfigurationCount();
			}
		}
		return count;
	}
	public MotifInstance findMotifByID(String type, long ID) {
		MotifInstanceMap mim = get(type);
		return mim.getByID(ID);
	}
	public Boolean containsComponentID(String type, long ID) {
		return (findMotifByID(type,ID) != null);
	}
	public String getTrace() {
		String trace = "";
		for(String motifType: keySet()) {
			trace += get(motifType).getTrace();
		}
		return trace;
	}
	public ArrayList<String> getMotifInstanceNames(){
		ArrayList<String> names = new ArrayList<String>();
		for(String type: keySet()) {
			MotifInstanceMap mimap = get(type);
			names.addAll(mimap.keySet());
		}
		return names;
	}
	public void evaluateEnabledStepsInMotifs() {
		for(String type: keySet()) {
			get(type).evaluateEnabledStep();
		}
	}
	public void remove(String type, String name) {
		MotifInstanceMap motifInstanceMap = get(type);
		motifInstanceMap.remove(name);
	}
	public MotifInstance getMotifInstance(String name) {
		for(String type: keySet()) {
			if(get(type).containsKey(name))
				return get(type).get(name);
		}
		System.err.println("reference to a motif name non existing");
		return null;
	}
	
	public boolean hasEnabledMotifInteraction() {
		for(String type: keySet()) 
			if(get(type).hasEnabledInteractions()) return true;
		return false;
	}
	public boolean hasEnabledMotifReconfiguration() {
		for(String type: keySet()) 
			if(get(type).hasEnabledReconfigurations()) return true;
		return false;
	}
	
	public String toString() {
		String str = "";
		for(String type: keySet()) {
			str += get(type).toString() + "\n";
		}
		return str;
	}
	
}
