package dybip.motifType;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;

import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;


import dybip.InteractionStmt.InteractionRuleStmtList;
import dybip.InteractionStmt.ReconfigurationRuleStmtList;
import dybip.systemStmt.motifTypeStmt.ComponentInstanceStmtList;
import dybip.systemStmt.motifTypeStmt.ReferStmtList;

public class DynamicMotifTypeGenerator {
	private static final String WORKINGDIRPATH = getCleanPath().replace("bin", "src"); // TODO : remove when done 
	private static final String BINARYCLASSNAME = "generated";
	private static final String GENERATEDCLASSDIRPATH = "generated/";
	
	public static void generate(String name){
		try {
			String pathStr = WORKINGDIRPATH + GENERATEDCLASSDIRPATH;
			new File(pathStr).mkdirs();
			
			
			File sourceFile   = new File(pathStr + name + ".java");
			FileWriter writer = new FileWriter(sourceFile);
			writer.write("package " + BINARYCLASSNAME + ";\n\nimport dybip.motifType.*;\nimport dybip.systemStmt.motifTypeStmt.*;\n import dybip.InteractionStmt.InteractionRuleStmtList;import dybip.InteractionStmt.ReconfigurationRuleStmtList;\n" +
													"\npublic class "+ name +" extends MotifType{\n" +
					 "\tpublic " + name + "(String name, ReferStmtList referals, ComponentInstanceStmtList instancesList, ReconfigurationRuleStmtList reconfigRuleList, InteractionRuleStmtList interactionList)\n\t{\n\t\tsuper(name, referals, instancesList, reconfigRuleList, interactionList);\n\t\t}\n}");
			writer.close();
			
			JavaCompiler compiler    = ToolProvider.getSystemJavaCompiler();
			// in - "standard" input; use System.in if null
		    // out - "standard" output; use System.out if null
		    // err - "standard" error; use System.err if null
		    // arguments - arguments to pass to the tool 
			if (compiler != null)
				compiler.run(null,null,null, sourceFile.getPath());
			
//			sourceFile.delete(); // TODO
			
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	public static MotifType createMotifType(String name, ReferStmtList referals, ComponentInstanceStmtList instancesList, ReconfigurationRuleStmtList reconfigList, InteractionRuleStmtList interactionList){
		// 1) update class loader to include new component type class created 
		// 2) load the new class
		
		MotifType mt = null;
		try {
			
			File root = new File(WORKINGDIRPATH);
			URL [] urls = new URL[] { root.toURI().toURL() };
			//			ClassLoader systemLoader = ClassLoader.getSystemClassLoader();
			ClassLoader systemLoader = DynamicMotifTypeGenerator.class.getClassLoader();
			URLClassLoader myClassLoader = new URLClassLoader(urls, systemLoader);
			
			
			String classNameToBeLoaded = BINARYCLASSNAME + "." + name;
			Class<?> classType = myClassLoader.loadClass(classNameToBeLoaded);
			Constructor<?> con = classType.getConstructor( String.class,  ReferStmtList.class, ComponentInstanceStmtList.class, ReconfigurationRuleStmtList.class, InteractionRuleStmtList.class);
			mt =  (MotifType) con.newInstance(name, referals, instancesList, reconfigList, interactionList);
			myClassLoader.close();
			
//			delete(new File(WORKINGDIRPATH + GENERATEDCLASSDIRPATH)); // TODO
			// TODO handle exceptions in a different way?
		} catch (MalformedURLException e) {
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}
		catch (NoSuchMethodException e) {
			e.printStackTrace();
		} 
		catch (SecurityException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return mt;
	}
	/**
	 * delete generated component type java class to avoid collision with different runs
	 * @param file
	 * @throws IOException
	 */
	public static void delete(File file)
			throws IOException{

		if(file.isDirectory()){

			//directory is empty, then delete it
			if(file.list().length==0){
				file.delete();
			}else{

				//list all the directory contents
				String files[] = file.list();

				for (String temp : files) {
					//construct the file structure
					File fileDelete = new File(file, temp);
					//recursive delete
					delete(fileDelete);
				}

				//check the directory again, if empty then delete it
				if(file.list().length==0){
					file.delete();
				}
			}

		}else{
			//if file, then delete it
			file.delete();
		}
	}
	private static String getCleanPath() {
		URL location = DynamicMotifTypeGenerator.class.getProtectionDomain().getCodeSource().getLocation();
		return location.getPath();
	}

}
