package dybip.system;

import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Random;
import java.util.Scanner;

import dybip.DyBIPObject;
import dybip.DyBIPRunner;
import dybip.MotifInstanceCountMap;
import dybip.ParameterMap;
import dybip.InteractionStmt.EnabledStepList;
import dybip.componentType.ComponentTypeMap;
import dybip.motifType.MotifTypeMap;
import dybip.systemStmt.SystemStmt;

public class SystemInstance extends DyBIPObject{
	private ComponentTypeMap ctmap;
	private MotifTypeMap mtmap;
	private SystemContext context;

	private Scanner scan = new Scanner(java.lang.System.in);
	

	private int totalExecutedInteractionCount;
	private int totalExecutedReconfigurationCount;
	private long totalExecutionTime;
	
	private Integer [] componentInstanceCount;
	private Integer [] motifInstanceCount;
	private Integer [] enabledInteractionCount;
	private Integer [] enabledReconfigurationCount;
	
//	private long avgComponentInstanceCount;
//	private long avgMotifInstanceCount;

	public SystemInstance(SystemStmt stmt, MotifInstanceCountMap micmap, ParameterMap pmap){
		ctmap = new ComponentTypeMap(stmt.getComponentTypeStmtList());
		mtmap = new MotifTypeMap(stmt.getMotifTypeStmtList());
		context = new SystemContext(stmt.getMotifInstanceStmtList(), micmap, pmap, mtmap, ctmap, stmt.getInterReconfigurationRuleStmtList());
		
		
		totalExecutedInteractionCount = 0;
		totalExecutedReconfigurationCount = 0;
		totalExecutionTime = 0;
	}
	
	public void run(int runCount, boolean moveMode, int moveCount, int toStringMode, PrintWriter statWriter, PrintWriter sizeWriter, boolean printStatsForThisRun){
		long startTime = System.currentTimeMillis();
		componentInstanceCount = new Integer  [moveCount];
		motifInstanceCount = new Integer  [moveCount];
		enabledInteractionCount = new Integer [moveCount];
		enabledReconfigurationCount = new Integer [moveCount];

		if(moveMode == DyBIPRunner.USERDRIVEN)
			for(int i = 0 ; i < moveCount; i++){
				context.evaluateEnabledSteps();
				context.printEnabledSteps();
				
				componentInstanceCount[i] = getComponentInstanceCount();
				motifInstanceCount[i] = getMotifInstanceCount();
				enabledInteractionCount[i] = getEnabledInteractionCount();
				enabledReconfigurationCount[i] = getEnabledReconfigurationCount();
				
				makeStep();
			}
		else if(moveMode == DyBIPRunner.RANDOM)
			for(int i = 0 ; i < moveCount; i++){
				context.evaluateEnabledSteps();
				context.printEnabledSteps();
				
				System.out.println("---Step " + i);
				componentInstanceCount[i] = getComponentInstanceCount();
				motifInstanceCount[i] = getMotifInstanceCount();
				enabledInteractionCount[i] = getEnabledInteractionCount();
				enabledReconfigurationCount[i] = getEnabledReconfigurationCount();
				
				makeRandomStep();
//				sizeWriter.print(context.trackMotifSize("Arena", i, runCount));
//				sizeWriter.print(context.trackMotifSize("ReateCoreTask,CoreTask,DeleteCoreTask", i, runCount));
//				sizeWriter.print(context.trackMotifSize("Processor", i, runCount));
				
				// System.err.println(toString());
			}
		long stopTime = System.currentTimeMillis();
		totalExecutionTime = stopTime - startTime;
		if(printStatsForThisRun) statWriter.println(SingleRunBenchmarkTable());
		if(printStatsForThisRun) statWriter.println(benchmarkTableHeader());
		statWriter.println(benchmarkTable());
		statWriter.flush();
		sizeWriter.flush();
	}
	public void makeStep(){
		// check enabled system steps
		if(context.hasEnabledInterReconfigurationStep()){	
			int stepIndex = scan.nextInt();
			context.Execute(stepIndex);
//			context.printChosenStep(stepIndex);
			totalExecutedReconfigurationCount++;
		}
		else if(context.hasEnabledMotifStep()) {
		
			String str = scan.nextLine();

			String[] array = str.split(" ");
			String motifInstanceName = array[0]; 
			String stepType = array[1];
			int stepIndex = Integer.parseInt(array[2]);

			if(stepType.equals("r") || stepType.equals("R")) totalExecutedReconfigurationCount++;
			else if(stepType.equals("i") || stepType.equals("I"))totalExecutedInteractionCount++;
			
			// assumption user enters correct and valid string names
			context.Execute(motifInstanceName, stepType, stepIndex);
//			context.printChosenStep(motifInstanceName, stepType, stepIndex);
			
			//----------System.err.println(toString());
		}

	}
	public void makeRandomStep(){
		int stepType = new Random().nextInt(2);
		// execute a inter-reconfiguration step
		if(!context.hasEnabledInterReconfigurationStep() && !context.hasEnabledMotifStep())
		{
			System.err.println("here");
		}
		if((context.hasEnabledInterReconfigurationStep() && !context.hasEnabledMotifStep()) || (stepType == 0 && context.hasEnabledInterReconfigurationStep())){
			EnabledStepList<?> enabledStepList = context.getEnabledInterReconfigurationStepList();	
			int  stepIndex = getRandomIndex(enabledStepList);
			context.printChosenStep(stepIndex);
			context.Execute(stepIndex);
			totalExecutedReconfigurationCount++;
		}
		// execute a motif step
		if((context.hasEnabledMotifStep() && !context.hasEnabledInterReconfigurationStep()) 
				|| (stepType == 1 && context.hasEnabledMotifStep())) {
			String motifInstanceName = getRandomInstanceName();
			String type = getRandomStepType();
			EnabledStepList<?> enabledStepList = context.getEnabledMotifStepList(motifInstanceName, type);

			// random chosen instance might not haver any enabled step
			// regenerate if needed
			while(enabledStepList.size() == 0) {
				motifInstanceName = getRandomInstanceName();
				type = getRandomStepType();
				enabledStepList = context.getEnabledMotifStepList(motifInstanceName, type);
			}
			int stepIndex = getRandomIndex(enabledStepList);

			if(type.equals("r") || type.equals("R"))  totalExecutedReconfigurationCount++;
			else if(type.equals("i") || type.equals("I"))totalExecutedInteractionCount++;
			
			context.printChosenStep(motifInstanceName, type, stepIndex);
			context.Execute(motifInstanceName, type, stepIndex);
		}
//		
	}
	private String getRandomInstanceName() {
		ArrayList<String> list =  context.getMotifInstanceNames();
		int randomIndex = new Random().nextInt(list.size());
		return list.get(randomIndex);
	}
	private String getRandomStepType() {
		int random = new Random().nextInt(2);
		if(random == 0)
			return "r";
		else 
			return "i";
	}
	// detailed stats of a single run of the system for 1000 steps
	public String SingleRunBenchmarkTable() {
		String str = SingleRunBenchmarkTableHeader();
		for(int i = 0; i < motifInstanceCount.length; i++) {
			str += i + "\t" +  componentInstanceCount[i] + "\t" + motifInstanceCount[i] + "\t" + enabledInteractionCount[i] + "\t" + enabledReconfigurationCount[i] + "\n";
		}
		
		str += "min\t"+ (int) Collections.min(Arrays.asList(componentInstanceCount))+ "\t"
						+ (int) Collections.min(Arrays.asList(motifInstanceCount)) + "\t"
						+ (int) Collections.min(Arrays.asList(enabledInteractionCount)) + "\t"
						+ (int) Collections.min(Arrays.asList(enabledReconfigurationCount)) + "\n";
		str += "max\t"+ (int) Collections.max(Arrays.asList(componentInstanceCount))+ "\t"
				+ (int) Collections.max(Arrays.asList(motifInstanceCount)) + "\t"
				+ (int) Collections.max(Arrays.asList(enabledInteractionCount)) + "\t"
				+ (int) Collections.max(Arrays.asList(enabledReconfigurationCount)) + "\n";
		return str += "intereactions:\t" + totalExecutedInteractionCount + "\n" + "reconfigurations:\t" + totalExecutedReconfigurationCount;
		
	}
	private String SingleRunBenchmarkTableHeader() {
		return "Step\tComponent Instance Count\tMotif Instance Count\tEnabled Interaction Count\t Enabled Reconfiguration Count\n";
	}
	
	public String benchmarkTable() {
		String benchMarkTable = motifInstanceCount[0] + "\t" + getAvgMotifInstanceCount() + "\t" +
						getMinMotifInstanceCount() + "\t" + getMaxMotifInstanceCount() + "\t" +
						componentInstanceCount[0] + "\t" + getAvgComponentInstanceCount() + "\t" +
						getMinComponentInstanceCount() + "\t" + getMaxComponentInstanceCount() + "\t" +
						getAvgEnabledInteractionCount() + "\t" + totalExecutedInteractionCount + "\t" +
						getAvgEnabledReconfigurationCount() + "\t" + totalExecutedReconfigurationCount + "\t" + 
						totalExecutionTime + "\t" + getRatioInteractionReconfig();
		return benchMarkTable;
	}
	private String benchmarkTableHeader() {
		return "Initial Motif Count\tAvg. Motif Count Per Step\t Min Motif Count\t Max Motif Count\t" + 
				"Initial Component Count\tAvg. Component Count Per Step\t Min Component Count\t Max Component Count\t" + 
				"Avg. Enabled Interaction Count Per Step\tTotal Executed Interaction Count\t" + 
				"Avg. Enabled Reconfiguration Count Per Step\tTotal Executed Reconfiguration Count\t" + 
				"Total Execution Time\t ratio interaction.reconfig";
	}
	private float getRatioInteractionReconfig() {
		return (float) totalExecutedInteractionCount/totalExecutedReconfigurationCount;
	}
	private int getMinMotifInstanceCount() {
		return (int) Collections.min(Arrays.asList(motifInstanceCount));
	}
	private int getMaxMotifInstanceCount() {
		return (int) Collections.max(Arrays.asList(motifInstanceCount));
	}
	private int getMinComponentInstanceCount() {
		return (int) Collections.min(Arrays.asList(componentInstanceCount));
	}
	private int getMaxComponentInstanceCount() {
		return (int) Collections.max(Arrays.asList(componentInstanceCount));
	}
	private int getComponentInstanceCount() {
		return context.getComponentInstanceCount();
	}
	private int getMotifInstanceCount() {
		return context.getMotifInstanceCount();
	}
	private int getEnabledReconfigurationCount() {
		return context.getEnabledReconfigurationCount();
	}
	private int getEnabledInteractionCount() {
		return context.getEnabledInteractionCount();
	}
	private float getAvgComponentInstanceCount() {
		float avg = 0;
		for(int i = 0; i < componentInstanceCount.length; i++)
			avg += componentInstanceCount[i];
		avg /= componentInstanceCount.length;
		return avg;
	}
	private float getAvgMotifInstanceCount() {
		float avg = 0;
		for(int i = 0; i < motifInstanceCount.length; i++)
			avg += motifInstanceCount[i];
		avg /= motifInstanceCount.length;
		return avg;
	}
	private float getAvgEnabledInteractionCount() {
		long avg = 0;
		for(int i = 0; i < enabledInteractionCount.length; i++)
			avg += enabledInteractionCount[i];
		avg /= enabledInteractionCount.length;
		return avg;
	}
	private float getAvgEnabledReconfigurationCount() {
		float avg = 0;
		for(int i = 0; i < enabledReconfigurationCount.length; i++)
			avg += enabledReconfigurationCount[i];
		avg /= enabledReconfigurationCount.length;
		return avg;
	}
	private int getRandomIndex(EnabledStepList<?> list){
		return new Random().nextInt(list.size());
	}
	public String getTrace() {
		return context.getTrace();
	}
	public String toString(){
		return "System Instance: \n" + 
				"\tComponent Type Map\n" + ctmap.toString() + "\n" +
				"\tMotif Type Map\n" + mtmap.toString() + "\n" +
				"\tContext\n" + context.toString() + "\n" ;

	}
}
