package dybip.systemStmt.componentTypeStmt;

import dybip.DyBIPObject;
import dybip.GuardActionPair;
/**
 * This class represents a transition in a component type. The transition takes a component from one state to another on some port.
 * Each transition has a guard and an action pair. The guard is an expression which must evaluate to true in order for the transition to happen
 * Therefore a transition is enabled if and only if the guard evaluates to true.
 * The action is a bunch of expressions that must be executed if the transition happens.
 * <br/>
 * Therefore a transition is identified by four things:
 * <ul>
 * <li> A start state </li>
 * <li> A to state </li>
 * <li> A port </li>
 * <li> A guard and action pair </li>
 * </ul>
 * @author el-ballouli
 *
 */
public class TransitionStmt extends DyBIPObject{
	StateStmt fromState;
	StateStmt toState;
	PortStmt port;
	GuardActionPair guardAction;
	
	/** 
	 * Constructs a new transition from the provided parameters
	 * @param fromState
	 * @param port
	 * @param toState
	 * @param pair
	 */
	public TransitionStmt(StateStmt fromState, PortStmt port, StateStmt toState, GuardActionPair pair){
		this.fromState = fromState;
		this.toState = toState;
		this.port = port;
		this.guardAction = pair;;
	}
	public GuardActionPair getGuardActionPair(){
		return guardAction;
	}
	/**
	 * @return returns the state from which the transition starts
	 */
	public StateStmt getFromState(){
		return fromState;
	}
	/**
	 * 
	 * @return return the state at which the transition ends
	 */
	public StateStmt getToState(){
		return toState;
	}
	/**
	 * 
	 * @return returns the port for this transition
	 */
	public PortStmt getPort(){
		return port;
	}
	/**
	 * @return returns a string conforming to the dybip language syntax for specifying a transition
	 */
	public String toString(){
		return "\t\ton " + port.toString() + " from " + fromState.toString() + " to " +  toState.toString() + " " + guardAction.toString(); 
	}
	
	public String toString(int toStringMode){
		return "";
	}
}
