package miscellaneous;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Arrays;
import java.util.List;
import java.util.Scanner;


public class PlatoonFigureGenerator {

	private static double xSep = 0.25;
	private static double ySep = 0.10;
	private static double barHeight = 0.1;
	//	private static String[] colors = {"SpringGreen", "OliveGreen" ,"Green", "SeaGreen",
	//								"RoyalBlue", "TealBlue", "SkyBlue",
	//								"Red", "Salmon", 
	//								"Orange", "Melon", "Apricot",
	//								"Yellow", "yellow!70", "Dandelion",
	//								"DarkOrchid", "Thistle",
	//								"Magenta", "Lavender", "Gray"};
	private static int startingStep = 400;
	private static int endStep = 1800;
	private static int interval = 100;


	private static final int RUN = 0;
	private static final int PLATOON = 1;
	private static final int STEP = 2;
	private static final int SIZE = 3;
	private static final int HEADPOS = 4;
	private static final int TRACKCARID = 130;
	private static final double TOTALPLATOON = 200;
	public static void main(String [] args) throws FileNotFoundException {		
		parseFile();
	}

	public static void parseFile() throws FileNotFoundException {
		Scanner fileScan = new Scanner(new File("examples/statistics/size_platoon3_s4001_r1_C1_F1_P1_Cn_n200 + 100.txt"));
		// skip header 8 line
		for(int i = 0; i < 8; i++) fileScan.nextLine();

		// skip until starting step
		String [] line = fileScan.nextLine().split("\t");
		int step = Integer.parseInt(line[STEP]);

		while(fileScan.hasNextLine() &&  step != startingStep) {
			line = fileScan.nextLine().split("\t");
			step = Integer.parseInt(line[STEP]);
		}

		Level levels = new Level();

		String [] platoonId = Arrays.copyOfRange(line, 5, line.length);
		Platoon p = new Platoon(line[SIZE], platoonId, line[HEADPOS]);

		PlatoonArrayList platoonList = new PlatoonArrayList(); 
		platoonList.add(p);
		levels.add(platoonList);


		while(step <= endStep) {
			// skip steps not in interval
			while(step%interval != 0 && step <= endStep) {
				line = fileScan.nextLine().split("\t");
				step = Integer.parseInt(line[STEP]);
			}

			// parse all lines with same step
			int currentStep = step;
			platoonList = new PlatoonArrayList();

			while(step == currentStep) {
				int size = Integer.parseInt(line[SIZE]);
				if(line.length > 5) {
					platoonId = Arrays.copyOfRange(line, 5, line.length);
					p = new Platoon(line[SIZE], platoonId, line[HEADPOS]);}
				else {
					platoonId = new String [0];
					p = new Platoon(line[SIZE], platoonId, "0");}

				platoonList.add(p);

				line = fileScan.nextLine().split("\t");
				step = Integer.parseInt(line[STEP]);
			}
			levels.add(platoonList);
		}
		justifiedLatexRectangle(levels);	
		fileScan.close();
	}

	public static void justifiedLatexRectangle(Level steps) {
		double maxSpacing = getMaxPlatoonCount(steps)*xSep; 
		double levels = (endStep - startingStep)/interval;
		double ymax = levels*barHeight + levels*ySep;
		System.out.println("\\begin{tikzpicture}");
		for(int level = 0; level < steps.size(); level++)
		{
			double xrectmin = 0;
			double yrectmin = ymax-(barHeight*level)-(ySep*(level-1));

			steps.get(level).sort();
			//			System.err.println(steps);
			PlatoonArrayList platoonsAtLevel = steps.get(level);

			//xSep = maxSpacing/(platoonsAtLevel.size()+1);
			//	xrectmin = xSep;
			int tempSize = 0;
			for(int i = 0; i < platoonsAtLevel.size(); i++) {
				
				if (platoonsAtLevel.get(i).getSize() != 0) {
					tempSize += platoonsAtLevel.get(i).getSize();
					
					double scale = (platoonsAtLevel.get(i).getSize()/TOTALPLATOON)*10;
					double xrectmax = xrectmin + scale;
					double yrectmax = yrectmin + barHeight;
					String str = getLatexRectangleStr("Black", xrectmin, yrectmin, xrectmax, yrectmax);
					System.out.println(str);
//					List<Car> cars = Arrays.asList(platoonsAtLevel.get(i).getCars());

					if(tempSize > TRACKCARID) { 
						
//						List<Integer> platoonCarIDs = (Arrays.asList(platoonsAtLevel.get(i).getIds()));
//						double positionOfCar = platoonCarIDs.indexOf(TRACKCARID);
						// draw red block in center
						//	if(positionOfCar < -1)
//						System.err.println(positionOfCar + " " + platoonCarIDs.size() + " " + (platoonCarIDs.size() - positionOfCar));
//						System.err.println((tempSize-TRACKCARID));
						double xredmin = xrectmax - (((tempSize-TRACKCARID))/TOTALPLATOON)*10; //- xrectmin)/2;
						double yredmin = yrectmin;
						double xredmax = xredmin + 0.2;
						double yredmax = yredmin + barHeight;
						System.out.println(getLatexRectangleStr("Yellow", xredmin, yredmin, xredmax, yredmax));
						tempSize = 0;
					}
					xrectmin = xrectmax + xSep;}
			}			
		}
		System.out.println("\\end{tikzpicture}");
	}
	public static int getMaxPlatoonCount(Level levels) {
		int max = 0;
		for(int i = 0; i < levels.size(); i++) {
			if(levels.get(i).size() > max) max = levels.get(i).size();
		}
		return max;
	}
	public static String getLatexRectangleStr(String color, double xmin, double ymin, double xmax, double ymax) {
		//double xtext = xmin + (xmax - xmin)/2;
		//double ytext = ymin + (ymax - ymin)/2;
		return "\\fill["+color+"] ("+xmin+","+ymin+") rectangle ("+xmax+","+ymax+");";
		//\\node at ("+xtext+","+ytext+") {\\scriptsize $P_{"+(int)platoon+"}$ "+(int)size+"};";

	}
}
